"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class StoreAndForward(google.protobuf.message.Message):
    """
    TODO: REPLACE
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    class _RequestResponse:
        ValueType = typing.NewType("ValueType", builtins.int)
        V: typing_extensions.TypeAlias = ValueType

    class _RequestResponseEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[StoreAndForward._RequestResponse.ValueType], builtins.type):
        DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
        UNSET: StoreAndForward._RequestResponse.ValueType  # 0
        """
        Unset/unused
        """
        ROUTER_ERROR: StoreAndForward._RequestResponse.ValueType  # 1
        """
        Router is an in error state.
        """
        ROUTER_HEARTBEAT: StoreAndForward._RequestResponse.ValueType  # 2
        """
        Router heartbeat
        """
        ROUTER_PING: StoreAndForward._RequestResponse.ValueType  # 3
        """
        Router has requested the client respond. This can work as a
        "are you there" message.
        """
        ROUTER_PONG: StoreAndForward._RequestResponse.ValueType  # 4
        """
        The response to a "Ping"
        """
        ROUTER_BUSY: StoreAndForward._RequestResponse.ValueType  # 5
        """
        Router is currently busy. Please try again later.
        """
        ROUTER_HISTORY: StoreAndForward._RequestResponse.ValueType  # 6
        """
        Router is responding to a request for history.
        """
        ROUTER_STATS: StoreAndForward._RequestResponse.ValueType  # 7
        """
        Router is responding to a request for stats.
        """
        ROUTER_TEXT_DIRECT: StoreAndForward._RequestResponse.ValueType  # 8
        """
        Router sends a text message from its history that was a direct message.
        """
        ROUTER_TEXT_BROADCAST: StoreAndForward._RequestResponse.ValueType  # 9
        """
        Router sends a text message from its history that was a broadcast.
        """
        CLIENT_ERROR: StoreAndForward._RequestResponse.ValueType  # 64
        """
        Client is an in error state.
        """
        CLIENT_HISTORY: StoreAndForward._RequestResponse.ValueType  # 65
        """
        Client has requested a replay from the router.
        """
        CLIENT_STATS: StoreAndForward._RequestResponse.ValueType  # 66
        """
        Client has requested stats from the router.
        """
        CLIENT_PING: StoreAndForward._RequestResponse.ValueType  # 67
        """
        Client has requested the router respond. This can work as a
        "are you there" message.
        """
        CLIENT_PONG: StoreAndForward._RequestResponse.ValueType  # 68
        """
        The response to a "Ping"
        """
        CLIENT_ABORT: StoreAndForward._RequestResponse.ValueType  # 106
        """
        Client has requested that the router abort processing the client's request
        """

    class RequestResponse(_RequestResponse, metaclass=_RequestResponseEnumTypeWrapper):
        """
        001 - 063 = From Router
        064 - 127 = From Client
        """

    UNSET: StoreAndForward.RequestResponse.ValueType  # 0
    """
    Unset/unused
    """
    ROUTER_ERROR: StoreAndForward.RequestResponse.ValueType  # 1
    """
    Router is an in error state.
    """
    ROUTER_HEARTBEAT: StoreAndForward.RequestResponse.ValueType  # 2
    """
    Router heartbeat
    """
    ROUTER_PING: StoreAndForward.RequestResponse.ValueType  # 3
    """
    Router has requested the client respond. This can work as a
    "are you there" message.
    """
    ROUTER_PONG: StoreAndForward.RequestResponse.ValueType  # 4
    """
    The response to a "Ping"
    """
    ROUTER_BUSY: StoreAndForward.RequestResponse.ValueType  # 5
    """
    Router is currently busy. Please try again later.
    """
    ROUTER_HISTORY: StoreAndForward.RequestResponse.ValueType  # 6
    """
    Router is responding to a request for history.
    """
    ROUTER_STATS: StoreAndForward.RequestResponse.ValueType  # 7
    """
    Router is responding to a request for stats.
    """
    ROUTER_TEXT_DIRECT: StoreAndForward.RequestResponse.ValueType  # 8
    """
    Router sends a text message from its history that was a direct message.
    """
    ROUTER_TEXT_BROADCAST: StoreAndForward.RequestResponse.ValueType  # 9
    """
    Router sends a text message from its history that was a broadcast.
    """
    CLIENT_ERROR: StoreAndForward.RequestResponse.ValueType  # 64
    """
    Client is an in error state.
    """
    CLIENT_HISTORY: StoreAndForward.RequestResponse.ValueType  # 65
    """
    Client has requested a replay from the router.
    """
    CLIENT_STATS: StoreAndForward.RequestResponse.ValueType  # 66
    """
    Client has requested stats from the router.
    """
    CLIENT_PING: StoreAndForward.RequestResponse.ValueType  # 67
    """
    Client has requested the router respond. This can work as a
    "are you there" message.
    """
    CLIENT_PONG: StoreAndForward.RequestResponse.ValueType  # 68
    """
    The response to a "Ping"
    """
    CLIENT_ABORT: StoreAndForward.RequestResponse.ValueType  # 106
    """
    Client has requested that the router abort processing the client's request
    """

    @typing.final
    class Statistics(google.protobuf.message.Message):
        """
        TODO: REPLACE
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        MESSAGES_TOTAL_FIELD_NUMBER: builtins.int
        MESSAGES_SAVED_FIELD_NUMBER: builtins.int
        MESSAGES_MAX_FIELD_NUMBER: builtins.int
        UP_TIME_FIELD_NUMBER: builtins.int
        REQUESTS_FIELD_NUMBER: builtins.int
        REQUESTS_HISTORY_FIELD_NUMBER: builtins.int
        HEARTBEAT_FIELD_NUMBER: builtins.int
        RETURN_MAX_FIELD_NUMBER: builtins.int
        RETURN_WINDOW_FIELD_NUMBER: builtins.int
        messages_total: builtins.int
        """
        Number of messages we have ever seen
        """
        messages_saved: builtins.int
        """
        Number of messages we have currently saved our history.
        """
        messages_max: builtins.int
        """
        Maximum number of messages we will save
        """
        up_time: builtins.int
        """
        Router uptime in seconds
        """
        requests: builtins.int
        """
        Number of times any client sent a request to the S&F.
        """
        requests_history: builtins.int
        """
        Number of times the history was requested.
        """
        heartbeat: builtins.bool
        """
        Is the heartbeat enabled on the server?
        """
        return_max: builtins.int
        """
        Maximum number of messages the server will return.
        """
        return_window: builtins.int
        """
        Maximum history window in minutes the server will return messages from.
        """
        def __init__(
            self,
            *,
            messages_total: builtins.int = ...,
            messages_saved: builtins.int = ...,
            messages_max: builtins.int = ...,
            up_time: builtins.int = ...,
            requests: builtins.int = ...,
            requests_history: builtins.int = ...,
            heartbeat: builtins.bool = ...,
            return_max: builtins.int = ...,
            return_window: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["heartbeat", b"heartbeat", "messages_max", b"messages_max", "messages_saved", b"messages_saved", "messages_total", b"messages_total", "requests", b"requests", "requests_history", b"requests_history", "return_max", b"return_max", "return_window", b"return_window", "up_time", b"up_time"]) -> None: ...

    @typing.final
    class History(google.protobuf.message.Message):
        """
        TODO: REPLACE
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        HISTORY_MESSAGES_FIELD_NUMBER: builtins.int
        WINDOW_FIELD_NUMBER: builtins.int
        LAST_REQUEST_FIELD_NUMBER: builtins.int
        history_messages: builtins.int
        """
        Number of that will be sent to the client
        """
        window: builtins.int
        """
        The window of messages that was used to filter the history client requested
        """
        last_request: builtins.int
        """
        Index in the packet history of the last message sent in a previous request to the server.
        Will be sent to the client before sending the history and can be set in a subsequent request to avoid getting packets the server already sent to the client.
        """
        def __init__(
            self,
            *,
            history_messages: builtins.int = ...,
            window: builtins.int = ...,
            last_request: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["history_messages", b"history_messages", "last_request", b"last_request", "window", b"window"]) -> None: ...

    @typing.final
    class Heartbeat(google.protobuf.message.Message):
        """
        TODO: REPLACE
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PERIOD_FIELD_NUMBER: builtins.int
        SECONDARY_FIELD_NUMBER: builtins.int
        period: builtins.int
        """
        Period in seconds that the heartbeat is sent out that will be sent to the client
        """
        secondary: builtins.int
        """
        If set, this is not the primary Store & Forward router on the mesh
        """
        def __init__(
            self,
            *,
            period: builtins.int = ...,
            secondary: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["period", b"period", "secondary", b"secondary"]) -> None: ...

    RR_FIELD_NUMBER: builtins.int
    STATS_FIELD_NUMBER: builtins.int
    HISTORY_FIELD_NUMBER: builtins.int
    HEARTBEAT_FIELD_NUMBER: builtins.int
    TEXT_FIELD_NUMBER: builtins.int
    rr: global___StoreAndForward.RequestResponse.ValueType
    """
    TODO: REPLACE
    """
    text: builtins.bytes
    """
    Text from history message.
    """
    @property
    def stats(self) -> global___StoreAndForward.Statistics:
        """
        TODO: REPLACE
        """

    @property
    def history(self) -> global___StoreAndForward.History:
        """
        TODO: REPLACE
        """

    @property
    def heartbeat(self) -> global___StoreAndForward.Heartbeat:
        """
        TODO: REPLACE
        """

    def __init__(
        self,
        *,
        rr: global___StoreAndForward.RequestResponse.ValueType = ...,
        stats: global___StoreAndForward.Statistics | None = ...,
        history: global___StoreAndForward.History | None = ...,
        heartbeat: global___StoreAndForward.Heartbeat | None = ...,
        text: builtins.bytes = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["heartbeat", b"heartbeat", "history", b"history", "stats", b"stats", "text", b"text", "variant", b"variant"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["heartbeat", b"heartbeat", "history", b"history", "rr", b"rr", "stats", b"stats", "text", b"text", "variant", b"variant"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["variant", b"variant"]) -> typing.Literal["stats", "history", "heartbeat", "text"] | None: ...

global___StoreAndForward = StoreAndForward



"""Support for logging from power meters/supplies."""

from .power_supply import PowerError, PowerMeter, PowerSupply
from .ppk2 import PPK2PowerSupply
from .riden import RidenPowerSupply
from .sim import SimPowerSupply
from .stress import PowerStress



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import meshtastic.protobuf.config_pb2
import meshtastic.protobuf.mesh_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ServiceEnvelope(google.protobuf.message.Message):
    """
    This message wraps a MeshPacket with extra metadata about the sender and how it arrived.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    PACKET_FIELD_NUMBER: builtins.int
    CHANNEL_ID_FIELD_NUMBER: builtins.int
    GATEWAY_ID_FIELD_NUMBER: builtins.int
    channel_id: builtins.str
    """
    The global channel ID it was sent on
    """
    gateway_id: builtins.str
    """
    The sending gateway node ID. Can we use this to authenticate/prevent fake
    nodeid impersonation for senders? - i.e. use gateway/mesh id (which is authenticated) + local node id as
    the globally trusted nodenum
    """
    @property
    def packet(self) -> meshtastic.protobuf.mesh_pb2.MeshPacket:
        """
        The (probably encrypted) packet
        """

    def __init__(
        self,
        *,
        packet: meshtastic.protobuf.mesh_pb2.MeshPacket | None = ...,
        channel_id: builtins.str = ...,
        gateway_id: builtins.str = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["packet", b"packet"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["channel_id", b"channel_id", "gateway_id", b"gateway_id", "packet", b"packet"]) -> None: ...

global___ServiceEnvelope = ServiceEnvelope

@typing.final
class MapReport(google.protobuf.message.Message):
    """
    Information about a node intended to be reported unencrypted to a map using MQTT.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LONG_NAME_FIELD_NUMBER: builtins.int
    SHORT_NAME_FIELD_NUMBER: builtins.int
    ROLE_FIELD_NUMBER: builtins.int
    HW_MODEL_FIELD_NUMBER: builtins.int
    FIRMWARE_VERSION_FIELD_NUMBER: builtins.int
    REGION_FIELD_NUMBER: builtins.int
    MODEM_PRESET_FIELD_NUMBER: builtins.int
    HAS_DEFAULT_CHANNEL_FIELD_NUMBER: builtins.int
    LATITUDE_I_FIELD_NUMBER: builtins.int
    LONGITUDE_I_FIELD_NUMBER: builtins.int
    ALTITUDE_FIELD_NUMBER: builtins.int
    POSITION_PRECISION_FIELD_NUMBER: builtins.int
    NUM_ONLINE_LOCAL_NODES_FIELD_NUMBER: builtins.int
    long_name: builtins.str
    """
    A full name for this user, i.e. "Kevin Hester"
    """
    short_name: builtins.str
    """
    A VERY short name, ideally two characters.
    Suitable for a tiny OLED screen
    """
    role: meshtastic.protobuf.config_pb2.Config.DeviceConfig.Role.ValueType
    """
    Role of the node that applies specific settings for a particular use-case
    """
    hw_model: meshtastic.protobuf.mesh_pb2.HardwareModel.ValueType
    """
    Hardware model of the node, i.e. T-Beam, Heltec V3, etc...
    """
    firmware_version: builtins.str
    """
    Device firmware version string
    """
    region: meshtastic.protobuf.config_pb2.Config.LoRaConfig.RegionCode.ValueType
    """
    The region code for the radio (US, CN, EU433, etc...)
    """
    modem_preset: meshtastic.protobuf.config_pb2.Config.LoRaConfig.ModemPreset.ValueType
    """
    Modem preset used by the radio (LongFast, MediumSlow, etc...)
    """
    has_default_channel: builtins.bool
    """
    Whether the node has a channel with default PSK and name (LongFast, MediumSlow, etc...)
    and it uses the default frequency slot given the region and modem preset.
    """
    latitude_i: builtins.int
    """
    Latitude: multiply by 1e-7 to get degrees in floating point
    """
    longitude_i: builtins.int
    """
    Longitude: multiply by 1e-7 to get degrees in floating point
    """
    altitude: builtins.int
    """
    Altitude in meters above MSL
    """
    position_precision: builtins.int
    """
    Indicates the bits of precision for latitude and longitude set by the sending node
    """
    num_online_local_nodes: builtins.int
    """
    Number of online nodes (heard in the last 2 hours) this node has in its list that were received locally (not via MQTT)
    """
    def __init__(
        self,
        *,
        long_name: builtins.str = ...,
        short_name: builtins.str = ...,
        role: meshtastic.protobuf.config_pb2.Config.DeviceConfig.Role.ValueType = ...,
        hw_model: meshtastic.protobuf.mesh_pb2.HardwareModel.ValueType = ...,
        firmware_version: builtins.str = ...,
        region: meshtastic.protobuf.config_pb2.Config.LoRaConfig.RegionCode.ValueType = ...,
        modem_preset: meshtastic.protobuf.config_pb2.Config.LoRaConfig.ModemPreset.ValueType = ...,
        has_default_channel: builtins.bool = ...,
        latitude_i: builtins.int = ...,
        longitude_i: builtins.int = ...,
        altitude: builtins.int = ...,
        position_precision: builtins.int = ...,
        num_online_local_nodes: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["altitude", b"altitude", "firmware_version", b"firmware_version", "has_default_channel", b"has_default_channel", "hw_model", b"hw_model", "latitude_i", b"latitude_i", "long_name", b"long_name", "longitude_i", b"longitude_i", "modem_preset", b"modem_preset", "num_online_local_nodes", b"num_online_local_nodes", "position_precision", b"position_precision", "region", b"region", "role", b"role", "short_name", b"short_name"]) -> None: ...

global___MapReport = MapReport



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import meshtastic.protobuf.device_ui_pb2
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class Config(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    @typing.final
    class DeviceConfig(google.protobuf.message.Message):
        """
        Configuration
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _Role:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _RoleEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.DeviceConfig._Role.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            CLIENT: Config.DeviceConfig._Role.ValueType  # 0
            """
            Description: App connected or stand alone messaging device.
            Technical Details: Default Role
            """
            CLIENT_MUTE: Config.DeviceConfig._Role.ValueType  # 1
            """
             Description: Device that does not forward packets from other devices.
            """
            ROUTER: Config.DeviceConfig._Role.ValueType  # 2
            """
            Description: Infrastructure node for extending network coverage by relaying messages. Visible in Nodes list.
            Technical Details: Mesh packets will prefer to be routed over this node. This node will not be used by client apps.
              The wifi radio and the oled screen will be put to sleep.
              This mode may still potentially have higher power usage due to it's preference in message rebroadcasting on the mesh.
            """
            ROUTER_CLIENT: Config.DeviceConfig._Role.ValueType  # 3
            """
            Description: Combination of both ROUTER and CLIENT. Not for mobile devices.
            Deprecated in v2.3.15 because improper usage is impacting public meshes: Use ROUTER or CLIENT instead.
            """
            REPEATER: Config.DeviceConfig._Role.ValueType  # 4
            """
            Description: Infrastructure node for extending network coverage by relaying messages with minimal overhead. Not visible in Nodes list.
            Technical Details: Mesh packets will simply be rebroadcasted over this node. Nodes configured with this role will not originate NodeInfo, Position, Telemetry
              or any other packet type. They will simply rebroadcast any mesh packets on the same frequency, channel num, spread factor, and coding rate.
            """
            TRACKER: Config.DeviceConfig._Role.ValueType  # 5
            """
            Description: Broadcasts GPS position packets as priority.
            Technical Details: Position Mesh packets will be prioritized higher and sent more frequently by default.
              When used in conjunction with power.is_power_saving = true, nodes will wake up,
              send position, and then sleep for position.position_broadcast_secs seconds.
            """
            SENSOR: Config.DeviceConfig._Role.ValueType  # 6
            """
            Description: Broadcasts telemetry packets as priority.
            Technical Details: Telemetry Mesh packets will be prioritized higher and sent more frequently by default.
              When used in conjunction with power.is_power_saving = true, nodes will wake up,
              send environment telemetry, and then sleep for telemetry.environment_update_interval seconds.
            """
            TAK: Config.DeviceConfig._Role.ValueType  # 7
            """
            Description: Optimized for ATAK system communication and reduces routine broadcasts.
            Technical Details: Used for nodes dedicated for connection to an ATAK EUD.
               Turns off many of the routine broadcasts to favor CoT packet stream
               from the Meshtastic ATAK plugin -> IMeshService -> Node
            """
            CLIENT_HIDDEN: Config.DeviceConfig._Role.ValueType  # 8
            """
            Description: Device that only broadcasts as needed for stealth or power savings.
            Technical Details: Used for nodes that "only speak when spoken to"
               Turns all of the routine broadcasts but allows for ad-hoc communication
               Still rebroadcasts, but with local only rebroadcast mode (known meshes only)
               Can be used for clandestine operation or to dramatically reduce airtime / power consumption
            """
            LOST_AND_FOUND: Config.DeviceConfig._Role.ValueType  # 9
            """
            Description: Broadcasts location as message to default channel regularly for to assist with device recovery.
            Technical Details: Used to automatically send a text message to the mesh
               with the current position of the device on a frequent interval:
               "I'm lost! Position: lat / long"
            """
            TAK_TRACKER: Config.DeviceConfig._Role.ValueType  # 10
            """
            Description: Enables automatic TAK PLI broadcasts and reduces routine broadcasts.
            Technical Details: Turns off many of the routine broadcasts to favor ATAK CoT packet stream
               and automatic TAK PLI (position location information) broadcasts.
               Uses position module configuration to determine TAK PLI broadcast interval.
            """
            ROUTER_LATE: Config.DeviceConfig._Role.ValueType  # 11
            """
            Description: Will always rebroadcast packets, but will do so after all other modes.
            Technical Details: Used for router nodes that are intended to provide additional coverage
               in areas not already covered by other routers, or to bridge around problematic terrain,
               but should not be given priority over other routers in order to avoid unnecessaraily
               consuming hops.
            """

        class Role(_Role, metaclass=_RoleEnumTypeWrapper):
            """
            Defines the device's role on the Mesh network
            """

        CLIENT: Config.DeviceConfig.Role.ValueType  # 0
        """
        Description: App connected or stand alone messaging device.
        Technical Details: Default Role
        """
        CLIENT_MUTE: Config.DeviceConfig.Role.ValueType  # 1
        """
         Description: Device that does not forward packets from other devices.
        """
        ROUTER: Config.DeviceConfig.Role.ValueType  # 2
        """
        Description: Infrastructure node for extending network coverage by relaying messages. Visible in Nodes list.
        Technical Details: Mesh packets will prefer to be routed over this node. This node will not be used by client apps.
          The wifi radio and the oled screen will be put to sleep.
          This mode may still potentially have higher power usage due to it's preference in message rebroadcasting on the mesh.
        """
        ROUTER_CLIENT: Config.DeviceConfig.Role.ValueType  # 3
        """
        Description: Combination of both ROUTER and CLIENT. Not for mobile devices.
        Deprecated in v2.3.15 because improper usage is impacting public meshes: Use ROUTER or CLIENT instead.
        """
        REPEATER: Config.DeviceConfig.Role.ValueType  # 4
        """
        Description: Infrastructure node for extending network coverage by relaying messages with minimal overhead. Not visible in Nodes list.
        Technical Details: Mesh packets will simply be rebroadcasted over this node. Nodes configured with this role will not originate NodeInfo, Position, Telemetry
          or any other packet type. They will simply rebroadcast any mesh packets on the same frequency, channel num, spread factor, and coding rate.
        """
        TRACKER: Config.DeviceConfig.Role.ValueType  # 5
        """
        Description: Broadcasts GPS position packets as priority.
        Technical Details: Position Mesh packets will be prioritized higher and sent more frequently by default.
          When used in conjunction with power.is_power_saving = true, nodes will wake up,
          send position, and then sleep for position.position_broadcast_secs seconds.
        """
        SENSOR: Config.DeviceConfig.Role.ValueType  # 6
        """
        Description: Broadcasts telemetry packets as priority.
        Technical Details: Telemetry Mesh packets will be prioritized higher and sent more frequently by default.
          When used in conjunction with power.is_power_saving = true, nodes will wake up,
          send environment telemetry, and then sleep for telemetry.environment_update_interval seconds.
        """
        TAK: Config.DeviceConfig.Role.ValueType  # 7
        """
        Description: Optimized for ATAK system communication and reduces routine broadcasts.
        Technical Details: Used for nodes dedicated for connection to an ATAK EUD.
           Turns off many of the routine broadcasts to favor CoT packet stream
           from the Meshtastic ATAK plugin -> IMeshService -> Node
        """
        CLIENT_HIDDEN: Config.DeviceConfig.Role.ValueType  # 8
        """
        Description: Device that only broadcasts as needed for stealth or power savings.
        Technical Details: Used for nodes that "only speak when spoken to"
           Turns all of the routine broadcasts but allows for ad-hoc communication
           Still rebroadcasts, but with local only rebroadcast mode (known meshes only)
           Can be used for clandestine operation or to dramatically reduce airtime / power consumption
        """
        LOST_AND_FOUND: Config.DeviceConfig.Role.ValueType  # 9
        """
        Description: Broadcasts location as message to default channel regularly for to assist with device recovery.
        Technical Details: Used to automatically send a text message to the mesh
           with the current position of the device on a frequent interval:
           "I'm lost! Position: lat / long"
        """
        TAK_TRACKER: Config.DeviceConfig.Role.ValueType  # 10
        """
        Description: Enables automatic TAK PLI broadcasts and reduces routine broadcasts.
        Technical Details: Turns off many of the routine broadcasts to favor ATAK CoT packet stream
           and automatic TAK PLI (position location information) broadcasts.
           Uses position module configuration to determine TAK PLI broadcast interval.
        """
        ROUTER_LATE: Config.DeviceConfig.Role.ValueType  # 11
        """
        Description: Will always rebroadcast packets, but will do so after all other modes.
        Technical Details: Used for router nodes that are intended to provide additional coverage
           in areas not already covered by other routers, or to bridge around problematic terrain,
           but should not be given priority over other routers in order to avoid unnecessaraily
           consuming hops.
        """

        class _RebroadcastMode:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _RebroadcastModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.DeviceConfig._RebroadcastMode.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            ALL: Config.DeviceConfig._RebroadcastMode.ValueType  # 0
            """
            Default behavior.
            Rebroadcast any observed message, if it was on our private channel or from another mesh with the same lora params.
            """
            ALL_SKIP_DECODING: Config.DeviceConfig._RebroadcastMode.ValueType  # 1
            """
            Same as behavior as ALL but skips packet decoding and simply rebroadcasts them.
            Only available in Repeater role. Setting this on any other roles will result in ALL behavior.
            """
            LOCAL_ONLY: Config.DeviceConfig._RebroadcastMode.ValueType  # 2
            """
            Ignores observed messages from foreign meshes that are open or those which it cannot decrypt.
            Only rebroadcasts message on the nodes local primary / secondary channels.
            """
            KNOWN_ONLY: Config.DeviceConfig._RebroadcastMode.ValueType  # 3
            """
            Ignores observed messages from foreign meshes like LOCAL_ONLY,
            but takes it step further by also ignoring messages from nodenums not in the node's known list (NodeDB)
            """
            NONE: Config.DeviceConfig._RebroadcastMode.ValueType  # 4
            """
            Only permitted for SENSOR, TRACKER and TAK_TRACKER roles, this will inhibit all rebroadcasts, not unlike CLIENT_MUTE role.
            """
            CORE_PORTNUMS_ONLY: Config.DeviceConfig._RebroadcastMode.ValueType  # 5
            """
            Ignores packets from non-standard portnums such as: TAK, RangeTest, PaxCounter, etc.
            Only rebroadcasts packets with standard portnums: NodeInfo, Text, Position, Telemetry, and Routing.
            """

        class RebroadcastMode(_RebroadcastMode, metaclass=_RebroadcastModeEnumTypeWrapper):
            """
            Defines the device's behavior for how messages are rebroadcast
            """

        ALL: Config.DeviceConfig.RebroadcastMode.ValueType  # 0
        """
        Default behavior.
        Rebroadcast any observed message, if it was on our private channel or from another mesh with the same lora params.
        """
        ALL_SKIP_DECODING: Config.DeviceConfig.RebroadcastMode.ValueType  # 1
        """
        Same as behavior as ALL but skips packet decoding and simply rebroadcasts them.
        Only available in Repeater role. Setting this on any other roles will result in ALL behavior.
        """
        LOCAL_ONLY: Config.DeviceConfig.RebroadcastMode.ValueType  # 2
        """
        Ignores observed messages from foreign meshes that are open or those which it cannot decrypt.
        Only rebroadcasts message on the nodes local primary / secondary channels.
        """
        KNOWN_ONLY: Config.DeviceConfig.RebroadcastMode.ValueType  # 3
        """
        Ignores observed messages from foreign meshes like LOCAL_ONLY,
        but takes it step further by also ignoring messages from nodenums not in the node's known list (NodeDB)
        """
        NONE: Config.DeviceConfig.RebroadcastMode.ValueType  # 4
        """
        Only permitted for SENSOR, TRACKER and TAK_TRACKER roles, this will inhibit all rebroadcasts, not unlike CLIENT_MUTE role.
        """
        CORE_PORTNUMS_ONLY: Config.DeviceConfig.RebroadcastMode.ValueType  # 5
        """
        Ignores packets from non-standard portnums such as: TAK, RangeTest, PaxCounter, etc.
        Only rebroadcasts packets with standard portnums: NodeInfo, Text, Position, Telemetry, and Routing.
        """

        ROLE_FIELD_NUMBER: builtins.int
        SERIAL_ENABLED_FIELD_NUMBER: builtins.int
        BUTTON_GPIO_FIELD_NUMBER: builtins.int
        BUZZER_GPIO_FIELD_NUMBER: builtins.int
        REBROADCAST_MODE_FIELD_NUMBER: builtins.int
        NODE_INFO_BROADCAST_SECS_FIELD_NUMBER: builtins.int
        DOUBLE_TAP_AS_BUTTON_PRESS_FIELD_NUMBER: builtins.int
        IS_MANAGED_FIELD_NUMBER: builtins.int
        DISABLE_TRIPLE_CLICK_FIELD_NUMBER: builtins.int
        TZDEF_FIELD_NUMBER: builtins.int
        LED_HEARTBEAT_DISABLED_FIELD_NUMBER: builtins.int
        role: global___Config.DeviceConfig.Role.ValueType
        """
        Sets the role of node
        """
        serial_enabled: builtins.bool
        """
        Disabling this will disable the SerialConsole by not initilizing the StreamAPI
        Moved to SecurityConfig
        """
        button_gpio: builtins.int
        """
        For boards without a hard wired button, this is the pin number that will be used
        Boards that have more than one button can swap the function with this one. defaults to BUTTON_PIN if defined.
        """
        buzzer_gpio: builtins.int
        """
        For boards without a PWM buzzer, this is the pin number that will be used
        Defaults to PIN_BUZZER if defined.
        """
        rebroadcast_mode: global___Config.DeviceConfig.RebroadcastMode.ValueType
        """
        Sets the role of node
        """
        node_info_broadcast_secs: builtins.int
        """
        Send our nodeinfo this often
        Defaults to 900 Seconds (15 minutes)
        """
        double_tap_as_button_press: builtins.bool
        """
        Treat double tap interrupt on supported accelerometers as a button press if set to true
        """
        is_managed: builtins.bool
        """
        If true, device is considered to be "managed" by a mesh administrator
        Clients should then limit available configuration and administrative options inside the user interface
        Moved to SecurityConfig
        """
        disable_triple_click: builtins.bool
        """
        Disables the triple-press of user button to enable or disable GPS
        """
        tzdef: builtins.str
        """
        POSIX Timezone definition string from https://github.com/nayarsystems/posix_tz_db/blob/master/zones.csv.
        """
        led_heartbeat_disabled: builtins.bool
        """
        If true, disable the default blinking LED (LED_PIN) behavior on the device
        """
        def __init__(
            self,
            *,
            role: global___Config.DeviceConfig.Role.ValueType = ...,
            serial_enabled: builtins.bool = ...,
            button_gpio: builtins.int = ...,
            buzzer_gpio: builtins.int = ...,
            rebroadcast_mode: global___Config.DeviceConfig.RebroadcastMode.ValueType = ...,
            node_info_broadcast_secs: builtins.int = ...,
            double_tap_as_button_press: builtins.bool = ...,
            is_managed: builtins.bool = ...,
            disable_triple_click: builtins.bool = ...,
            tzdef: builtins.str = ...,
            led_heartbeat_disabled: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["button_gpio", b"button_gpio", "buzzer_gpio", b"buzzer_gpio", "disable_triple_click", b"disable_triple_click", "double_tap_as_button_press", b"double_tap_as_button_press", "is_managed", b"is_managed", "led_heartbeat_disabled", b"led_heartbeat_disabled", "node_info_broadcast_secs", b"node_info_broadcast_secs", "rebroadcast_mode", b"rebroadcast_mode", "role", b"role", "serial_enabled", b"serial_enabled", "tzdef", b"tzdef"]) -> None: ...

    @typing.final
    class PositionConfig(google.protobuf.message.Message):
        """
        Position Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _PositionFlags:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _PositionFlagsEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.PositionConfig._PositionFlags.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            UNSET: Config.PositionConfig._PositionFlags.ValueType  # 0
            """
            Required for compilation
            """
            ALTITUDE: Config.PositionConfig._PositionFlags.ValueType  # 1
            """
            Include an altitude value (if available)
            """
            ALTITUDE_MSL: Config.PositionConfig._PositionFlags.ValueType  # 2
            """
            Altitude value is MSL
            """
            GEOIDAL_SEPARATION: Config.PositionConfig._PositionFlags.ValueType  # 4
            """
            Include geoidal separation
            """
            DOP: Config.PositionConfig._PositionFlags.ValueType  # 8
            """
            Include the DOP value ; PDOP used by default, see below
            """
            HVDOP: Config.PositionConfig._PositionFlags.ValueType  # 16
            """
            If POS_DOP set, send separate HDOP / VDOP values instead of PDOP
            """
            SATINVIEW: Config.PositionConfig._PositionFlags.ValueType  # 32
            """
            Include number of "satellites in view"
            """
            SEQ_NO: Config.PositionConfig._PositionFlags.ValueType  # 64
            """
            Include a sequence number incremented per packet
            """
            TIMESTAMP: Config.PositionConfig._PositionFlags.ValueType  # 128
            """
            Include positional timestamp (from GPS solution)
            """
            HEADING: Config.PositionConfig._PositionFlags.ValueType  # 256
            """
            Include positional heading
            Intended for use with vehicle not walking speeds
            walking speeds are likely to be error prone like the compass
            """
            SPEED: Config.PositionConfig._PositionFlags.ValueType  # 512
            """
            Include positional speed
            Intended for use with vehicle not walking speeds
            walking speeds are likely to be error prone like the compass
            """

        class PositionFlags(_PositionFlags, metaclass=_PositionFlagsEnumTypeWrapper):
            """
            Bit field of boolean configuration options, indicating which optional
            fields to include when assembling POSITION messages.
            Longitude, latitude, altitude, speed, heading, and DOP
            are always included (also time if GPS-synced)
            NOTE: the more fields are included, the larger the message will be -
              leading to longer airtime and a higher risk of packet loss
            """

        UNSET: Config.PositionConfig.PositionFlags.ValueType  # 0
        """
        Required for compilation
        """
        ALTITUDE: Config.PositionConfig.PositionFlags.ValueType  # 1
        """
        Include an altitude value (if available)
        """
        ALTITUDE_MSL: Config.PositionConfig.PositionFlags.ValueType  # 2
        """
        Altitude value is MSL
        """
        GEOIDAL_SEPARATION: Config.PositionConfig.PositionFlags.ValueType  # 4
        """
        Include geoidal separation
        """
        DOP: Config.PositionConfig.PositionFlags.ValueType  # 8
        """
        Include the DOP value ; PDOP used by default, see below
        """
        HVDOP: Config.PositionConfig.PositionFlags.ValueType  # 16
        """
        If POS_DOP set, send separate HDOP / VDOP values instead of PDOP
        """
        SATINVIEW: Config.PositionConfig.PositionFlags.ValueType  # 32
        """
        Include number of "satellites in view"
        """
        SEQ_NO: Config.PositionConfig.PositionFlags.ValueType  # 64
        """
        Include a sequence number incremented per packet
        """
        TIMESTAMP: Config.PositionConfig.PositionFlags.ValueType  # 128
        """
        Include positional timestamp (from GPS solution)
        """
        HEADING: Config.PositionConfig.PositionFlags.ValueType  # 256
        """
        Include positional heading
        Intended for use with vehicle not walking speeds
        walking speeds are likely to be error prone like the compass
        """
        SPEED: Config.PositionConfig.PositionFlags.ValueType  # 512
        """
        Include positional speed
        Intended for use with vehicle not walking speeds
        walking speeds are likely to be error prone like the compass
        """

        class _GpsMode:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _GpsModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.PositionConfig._GpsMode.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            DISABLED: Config.PositionConfig._GpsMode.ValueType  # 0
            """
            GPS is present but disabled
            """
            ENABLED: Config.PositionConfig._GpsMode.ValueType  # 1
            """
            GPS is present and enabled
            """
            NOT_PRESENT: Config.PositionConfig._GpsMode.ValueType  # 2
            """
            GPS is not present on the device
            """

        class GpsMode(_GpsMode, metaclass=_GpsModeEnumTypeWrapper): ...
        DISABLED: Config.PositionConfig.GpsMode.ValueType  # 0
        """
        GPS is present but disabled
        """
        ENABLED: Config.PositionConfig.GpsMode.ValueType  # 1
        """
        GPS is present and enabled
        """
        NOT_PRESENT: Config.PositionConfig.GpsMode.ValueType  # 2
        """
        GPS is not present on the device
        """

        POSITION_BROADCAST_SECS_FIELD_NUMBER: builtins.int
        POSITION_BROADCAST_SMART_ENABLED_FIELD_NUMBER: builtins.int
        FIXED_POSITION_FIELD_NUMBER: builtins.int
        GPS_ENABLED_FIELD_NUMBER: builtins.int
        GPS_UPDATE_INTERVAL_FIELD_NUMBER: builtins.int
        GPS_ATTEMPT_TIME_FIELD_NUMBER: builtins.int
        POSITION_FLAGS_FIELD_NUMBER: builtins.int
        RX_GPIO_FIELD_NUMBER: builtins.int
        TX_GPIO_FIELD_NUMBER: builtins.int
        BROADCAST_SMART_MINIMUM_DISTANCE_FIELD_NUMBER: builtins.int
        BROADCAST_SMART_MINIMUM_INTERVAL_SECS_FIELD_NUMBER: builtins.int
        GPS_EN_GPIO_FIELD_NUMBER: builtins.int
        GPS_MODE_FIELD_NUMBER: builtins.int
        position_broadcast_secs: builtins.int
        """
        We should send our position this often (but only if it has changed significantly)
        Defaults to 15 minutes
        """
        position_broadcast_smart_enabled: builtins.bool
        """
        Adaptive position braoadcast, which is now the default.
        """
        fixed_position: builtins.bool
        """
        If set, this node is at a fixed position.
        We will generate GPS position updates at the regular interval, but use whatever the last lat/lon/alt we have for the node.
        The lat/lon/alt can be set by an internal GPS or with the help of the app.
        """
        gps_enabled: builtins.bool
        """
        Is GPS enabled for this node?
        """
        gps_update_interval: builtins.int
        """
        How often should we try to get GPS position (in seconds)
        or zero for the default of once every 30 seconds
        or a very large value (maxint) to update only once at boot.
        """
        gps_attempt_time: builtins.int
        """
        Deprecated in favor of using smart / regular broadcast intervals as implicit attempt time
        """
        position_flags: builtins.int
        """
        Bit field of boolean configuration options for POSITION messages
        (bitwise OR of PositionFlags)
        """
        rx_gpio: builtins.int
        """
        (Re)define GPS_RX_PIN for your board.
        """
        tx_gpio: builtins.int
        """
        (Re)define GPS_TX_PIN for your board.
        """
        broadcast_smart_minimum_distance: builtins.int
        """
        The minimum distance in meters traveled (since the last send) before we can send a position to the mesh if position_broadcast_smart_enabled
        """
        broadcast_smart_minimum_interval_secs: builtins.int
        """
        The minimum number of seconds (since the last send) before we can send a position to the mesh if position_broadcast_smart_enabled
        """
        gps_en_gpio: builtins.int
        """
        (Re)define PIN_GPS_EN for your board.
        """
        gps_mode: global___Config.PositionConfig.GpsMode.ValueType
        """
        Set where GPS is enabled, disabled, or not present
        """
        def __init__(
            self,
            *,
            position_broadcast_secs: builtins.int = ...,
            position_broadcast_smart_enabled: builtins.bool = ...,
            fixed_position: builtins.bool = ...,
            gps_enabled: builtins.bool = ...,
            gps_update_interval: builtins.int = ...,
            gps_attempt_time: builtins.int = ...,
            position_flags: builtins.int = ...,
            rx_gpio: builtins.int = ...,
            tx_gpio: builtins.int = ...,
            broadcast_smart_minimum_distance: builtins.int = ...,
            broadcast_smart_minimum_interval_secs: builtins.int = ...,
            gps_en_gpio: builtins.int = ...,
            gps_mode: global___Config.PositionConfig.GpsMode.ValueType = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["broadcast_smart_minimum_distance", b"broadcast_smart_minimum_distance", "broadcast_smart_minimum_interval_secs", b"broadcast_smart_minimum_interval_secs", "fixed_position", b"fixed_position", "gps_attempt_time", b"gps_attempt_time", "gps_en_gpio", b"gps_en_gpio", "gps_enabled", b"gps_enabled", "gps_mode", b"gps_mode", "gps_update_interval", b"gps_update_interval", "position_broadcast_secs", b"position_broadcast_secs", "position_broadcast_smart_enabled", b"position_broadcast_smart_enabled", "position_flags", b"position_flags", "rx_gpio", b"rx_gpio", "tx_gpio", b"tx_gpio"]) -> None: ...

    @typing.final
    class PowerConfig(google.protobuf.message.Message):
        """
        Power Config\\
        See [Power Config](/docs/settings/config/power) for additional power config details.
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        IS_POWER_SAVING_FIELD_NUMBER: builtins.int
        ON_BATTERY_SHUTDOWN_AFTER_SECS_FIELD_NUMBER: builtins.int
        ADC_MULTIPLIER_OVERRIDE_FIELD_NUMBER: builtins.int
        WAIT_BLUETOOTH_SECS_FIELD_NUMBER: builtins.int
        SDS_SECS_FIELD_NUMBER: builtins.int
        LS_SECS_FIELD_NUMBER: builtins.int
        MIN_WAKE_SECS_FIELD_NUMBER: builtins.int
        DEVICE_BATTERY_INA_ADDRESS_FIELD_NUMBER: builtins.int
        POWERMON_ENABLES_FIELD_NUMBER: builtins.int
        is_power_saving: builtins.bool
        """
        Description: Will sleep everything as much as possible, for the tracker and sensor role this will also include the lora radio. 
        Don't use this setting if you want to use your device with the phone apps or are using a device without a user button.
        Technical Details: Works for ESP32 devices and NRF52 devices in the Sensor or Tracker roles
        """
        on_battery_shutdown_after_secs: builtins.int
        """
         Description: If non-zero, the device will fully power off this many seconds after external power is removed.
        """
        adc_multiplier_override: builtins.float
        """
        Ratio of voltage divider for battery pin eg. 3.20 (R1=100k, R2=220k)
        Overrides the ADC_MULTIPLIER defined in variant for battery voltage calculation.
        https://meshtastic.org/docs/configuration/radio/power/#adc-multiplier-override
        Should be set to floating point value between 2 and 6
        """
        wait_bluetooth_secs: builtins.int
        """
         Description: The number of seconds for to wait before turning off BLE in No Bluetooth states
         Technical Details: ESP32 Only 0 for default of 1 minute
        """
        sds_secs: builtins.int
        """
        Super Deep Sleep Seconds
        While in Light Sleep if mesh_sds_timeout_secs is exceeded we will lower into super deep sleep
        for this value (default 1 year) or a button press
        0 for default of one year
        """
        ls_secs: builtins.int
        """
        Description: In light sleep the CPU is suspended, LoRa radio is on, BLE is off an GPS is on
        Technical Details: ESP32 Only 0 for default of 300
        """
        min_wake_secs: builtins.int
        """
        Description: While in light sleep when we receive packets on the LoRa radio we will wake and handle them and stay awake in no BLE mode for this value
        Technical Details: ESP32 Only 0 for default of 10 seconds
        """
        device_battery_ina_address: builtins.int
        """
        I2C address of INA_2XX to use for reading device battery voltage
        """
        powermon_enables: builtins.int
        """
        If non-zero, we want powermon log outputs.  With the particular (bitfield) sources enabled.
        Note: we picked an ID of 32 so that lower more efficient IDs can be used for more frequently used options.
        """
        def __init__(
            self,
            *,
            is_power_saving: builtins.bool = ...,
            on_battery_shutdown_after_secs: builtins.int = ...,
            adc_multiplier_override: builtins.float = ...,
            wait_bluetooth_secs: builtins.int = ...,
            sds_secs: builtins.int = ...,
            ls_secs: builtins.int = ...,
            min_wake_secs: builtins.int = ...,
            device_battery_ina_address: builtins.int = ...,
            powermon_enables: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["adc_multiplier_override", b"adc_multiplier_override", "device_battery_ina_address", b"device_battery_ina_address", "is_power_saving", b"is_power_saving", "ls_secs", b"ls_secs", "min_wake_secs", b"min_wake_secs", "on_battery_shutdown_after_secs", b"on_battery_shutdown_after_secs", "powermon_enables", b"powermon_enables", "sds_secs", b"sds_secs", "wait_bluetooth_secs", b"wait_bluetooth_secs"]) -> None: ...

    @typing.final
    class NetworkConfig(google.protobuf.message.Message):
        """
        Network Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _AddressMode:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _AddressModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.NetworkConfig._AddressMode.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            DHCP: Config.NetworkConfig._AddressMode.ValueType  # 0
            """
            obtain ip address via DHCP
            """
            STATIC: Config.NetworkConfig._AddressMode.ValueType  # 1
            """
            use static ip address
            """

        class AddressMode(_AddressMode, metaclass=_AddressModeEnumTypeWrapper): ...
        DHCP: Config.NetworkConfig.AddressMode.ValueType  # 0
        """
        obtain ip address via DHCP
        """
        STATIC: Config.NetworkConfig.AddressMode.ValueType  # 1
        """
        use static ip address
        """

        class _ProtocolFlags:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _ProtocolFlagsEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.NetworkConfig._ProtocolFlags.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            NO_BROADCAST: Config.NetworkConfig._ProtocolFlags.ValueType  # 0
            """
            Do not broadcast packets over any network protocol
            """
            UDP_BROADCAST: Config.NetworkConfig._ProtocolFlags.ValueType  # 1
            """
            Enable broadcasting packets via UDP over the local network
            """

        class ProtocolFlags(_ProtocolFlags, metaclass=_ProtocolFlagsEnumTypeWrapper):
            """
            Available flags auxiliary network protocols
            """

        NO_BROADCAST: Config.NetworkConfig.ProtocolFlags.ValueType  # 0
        """
        Do not broadcast packets over any network protocol
        """
        UDP_BROADCAST: Config.NetworkConfig.ProtocolFlags.ValueType  # 1
        """
        Enable broadcasting packets via UDP over the local network
        """

        @typing.final
        class IpV4Config(google.protobuf.message.Message):
            DESCRIPTOR: google.protobuf.descriptor.Descriptor

            IP_FIELD_NUMBER: builtins.int
            GATEWAY_FIELD_NUMBER: builtins.int
            SUBNET_FIELD_NUMBER: builtins.int
            DNS_FIELD_NUMBER: builtins.int
            ip: builtins.int
            """
            Static IP address
            """
            gateway: builtins.int
            """
            Static gateway address
            """
            subnet: builtins.int
            """
            Static subnet mask
            """
            dns: builtins.int
            """
            Static DNS server address
            """
            def __init__(
                self,
                *,
                ip: builtins.int = ...,
                gateway: builtins.int = ...,
                subnet: builtins.int = ...,
                dns: builtins.int = ...,
            ) -> None: ...
            def ClearField(self, field_name: typing.Literal["dns", b"dns", "gateway", b"gateway", "ip", b"ip", "subnet", b"subnet"]) -> None: ...

        WIFI_ENABLED_FIELD_NUMBER: builtins.int
        WIFI_SSID_FIELD_NUMBER: builtins.int
        WIFI_PSK_FIELD_NUMBER: builtins.int
        NTP_SERVER_FIELD_NUMBER: builtins.int
        ETH_ENABLED_FIELD_NUMBER: builtins.int
        ADDRESS_MODE_FIELD_NUMBER: builtins.int
        IPV4_CONFIG_FIELD_NUMBER: builtins.int
        RSYSLOG_SERVER_FIELD_NUMBER: builtins.int
        ENABLED_PROTOCOLS_FIELD_NUMBER: builtins.int
        wifi_enabled: builtins.bool
        """
        Enable WiFi (disables Bluetooth)
        """
        wifi_ssid: builtins.str
        """
        If set, this node will try to join the specified wifi network and
        acquire an address via DHCP
        """
        wifi_psk: builtins.str
        """
        If set, will be use to authenticate to the named wifi
        """
        ntp_server: builtins.str
        """
        NTP server to use if WiFi is conneced, defaults to `0.pool.ntp.org`
        """
        eth_enabled: builtins.bool
        """
        Enable Ethernet
        """
        address_mode: global___Config.NetworkConfig.AddressMode.ValueType
        """
        acquire an address via DHCP or assign static
        """
        rsyslog_server: builtins.str
        """
        rsyslog Server and Port
        """
        enabled_protocols: builtins.int
        """
        Flags for enabling/disabling network protocols
        """
        @property
        def ipv4_config(self) -> global___Config.NetworkConfig.IpV4Config:
            """
            struct to keep static address
            """

        def __init__(
            self,
            *,
            wifi_enabled: builtins.bool = ...,
            wifi_ssid: builtins.str = ...,
            wifi_psk: builtins.str = ...,
            ntp_server: builtins.str = ...,
            eth_enabled: builtins.bool = ...,
            address_mode: global___Config.NetworkConfig.AddressMode.ValueType = ...,
            ipv4_config: global___Config.NetworkConfig.IpV4Config | None = ...,
            rsyslog_server: builtins.str = ...,
            enabled_protocols: builtins.int = ...,
        ) -> None: ...
        def HasField(self, field_name: typing.Literal["ipv4_config", b"ipv4_config"]) -> builtins.bool: ...
        def ClearField(self, field_name: typing.Literal["address_mode", b"address_mode", "enabled_protocols", b"enabled_protocols", "eth_enabled", b"eth_enabled", "ipv4_config", b"ipv4_config", "ntp_server", b"ntp_server", "rsyslog_server", b"rsyslog_server", "wifi_enabled", b"wifi_enabled", "wifi_psk", b"wifi_psk", "wifi_ssid", b"wifi_ssid"]) -> None: ...

    @typing.final
    class DisplayConfig(google.protobuf.message.Message):
        """
        Display Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _GpsCoordinateFormat:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _GpsCoordinateFormatEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.DisplayConfig._GpsCoordinateFormat.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            DEC: Config.DisplayConfig._GpsCoordinateFormat.ValueType  # 0
            """
            GPS coordinates are displayed in the normal decimal degrees format:
            DD.DDDDDD DDD.DDDDDD
            """
            DMS: Config.DisplayConfig._GpsCoordinateFormat.ValueType  # 1
            """
            GPS coordinates are displayed in the degrees minutes seconds format:
            DD°MM'SS"C DDD°MM'SS"C, where C is the compass point representing the locations quadrant
            """
            UTM: Config.DisplayConfig._GpsCoordinateFormat.ValueType  # 2
            """
            Universal Transverse Mercator format:
            ZZB EEEEEE NNNNNNN, where Z is zone, B is band, E is easting, N is northing
            """
            MGRS: Config.DisplayConfig._GpsCoordinateFormat.ValueType  # 3
            """
            Military Grid Reference System format:
            ZZB CD EEEEE NNNNN, where Z is zone, B is band, C is the east 100k square, D is the north 100k square,
            E is easting, N is northing
            """
            OLC: Config.DisplayConfig._GpsCoordinateFormat.ValueType  # 4
            """
            Open Location Code (aka Plus Codes).
            """
            OSGR: Config.DisplayConfig._GpsCoordinateFormat.ValueType  # 5
            """
            Ordnance Survey Grid Reference (the National Grid System of the UK).
            Format: AB EEEEE NNNNN, where A is the east 100k square, B is the north 100k square,
            E is the easting, N is the northing
            """

        class GpsCoordinateFormat(_GpsCoordinateFormat, metaclass=_GpsCoordinateFormatEnumTypeWrapper):
            """
            How the GPS coordinates are displayed on the OLED screen.
            """

        DEC: Config.DisplayConfig.GpsCoordinateFormat.ValueType  # 0
        """
        GPS coordinates are displayed in the normal decimal degrees format:
        DD.DDDDDD DDD.DDDDDD
        """
        DMS: Config.DisplayConfig.GpsCoordinateFormat.ValueType  # 1
        """
        GPS coordinates are displayed in the degrees minutes seconds format:
        DD°MM'SS"C DDD°MM'SS"C, where C is the compass point representing the locations quadrant
        """
        UTM: Config.DisplayConfig.GpsCoordinateFormat.ValueType  # 2
        """
        Universal Transverse Mercator format:
        ZZB EEEEEE NNNNNNN, where Z is zone, B is band, E is easting, N is northing
        """
        MGRS: Config.DisplayConfig.GpsCoordinateFormat.ValueType  # 3
        """
        Military Grid Reference System format:
        ZZB CD EEEEE NNNNN, where Z is zone, B is band, C is the east 100k square, D is the north 100k square,
        E is easting, N is northing
        """
        OLC: Config.DisplayConfig.GpsCoordinateFormat.ValueType  # 4
        """
        Open Location Code (aka Plus Codes).
        """
        OSGR: Config.DisplayConfig.GpsCoordinateFormat.ValueType  # 5
        """
        Ordnance Survey Grid Reference (the National Grid System of the UK).
        Format: AB EEEEE NNNNN, where A is the east 100k square, B is the north 100k square,
        E is the easting, N is the northing
        """

        class _DisplayUnits:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _DisplayUnitsEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.DisplayConfig._DisplayUnits.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            METRIC: Config.DisplayConfig._DisplayUnits.ValueType  # 0
            """
            Metric (Default)
            """
            IMPERIAL: Config.DisplayConfig._DisplayUnits.ValueType  # 1
            """
            Imperial
            """

        class DisplayUnits(_DisplayUnits, metaclass=_DisplayUnitsEnumTypeWrapper):
            """
            Unit display preference
            """

        METRIC: Config.DisplayConfig.DisplayUnits.ValueType  # 0
        """
        Metric (Default)
        """
        IMPERIAL: Config.DisplayConfig.DisplayUnits.ValueType  # 1
        """
        Imperial
        """

        class _OledType:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _OledTypeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.DisplayConfig._OledType.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            OLED_AUTO: Config.DisplayConfig._OledType.ValueType  # 0
            """
            Default / Auto
            """
            OLED_SSD1306: Config.DisplayConfig._OledType.ValueType  # 1
            """
            Default / Auto
            """
            OLED_SH1106: Config.DisplayConfig._OledType.ValueType  # 2
            """
            Default / Auto
            """
            OLED_SH1107: Config.DisplayConfig._OledType.ValueType  # 3
            """
            Can not be auto detected but set by proto. Used for 128x128 screens
            """

        class OledType(_OledType, metaclass=_OledTypeEnumTypeWrapper):
            """
            Override OLED outo detect with this if it fails.
            """

        OLED_AUTO: Config.DisplayConfig.OledType.ValueType  # 0
        """
        Default / Auto
        """
        OLED_SSD1306: Config.DisplayConfig.OledType.ValueType  # 1
        """
        Default / Auto
        """
        OLED_SH1106: Config.DisplayConfig.OledType.ValueType  # 2
        """
        Default / Auto
        """
        OLED_SH1107: Config.DisplayConfig.OledType.ValueType  # 3
        """
        Can not be auto detected but set by proto. Used for 128x128 screens
        """

        class _DisplayMode:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _DisplayModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.DisplayConfig._DisplayMode.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            DEFAULT: Config.DisplayConfig._DisplayMode.ValueType  # 0
            """
            Default. The old style for the 128x64 OLED screen
            """
            TWOCOLOR: Config.DisplayConfig._DisplayMode.ValueType  # 1
            """
            Rearrange display elements to cater for bicolor OLED displays
            """
            INVERTED: Config.DisplayConfig._DisplayMode.ValueType  # 2
            """
            Same as TwoColor, but with inverted top bar. Not so good for Epaper displays
            """
            COLOR: Config.DisplayConfig._DisplayMode.ValueType  # 3
            """
            TFT Full Color Displays (not implemented yet)
            """

        class DisplayMode(_DisplayMode, metaclass=_DisplayModeEnumTypeWrapper): ...
        DEFAULT: Config.DisplayConfig.DisplayMode.ValueType  # 0
        """
        Default. The old style for the 128x64 OLED screen
        """
        TWOCOLOR: Config.DisplayConfig.DisplayMode.ValueType  # 1
        """
        Rearrange display elements to cater for bicolor OLED displays
        """
        INVERTED: Config.DisplayConfig.DisplayMode.ValueType  # 2
        """
        Same as TwoColor, but with inverted top bar. Not so good for Epaper displays
        """
        COLOR: Config.DisplayConfig.DisplayMode.ValueType  # 3
        """
        TFT Full Color Displays (not implemented yet)
        """

        class _CompassOrientation:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _CompassOrientationEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.DisplayConfig._CompassOrientation.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            DEGREES_0: Config.DisplayConfig._CompassOrientation.ValueType  # 0
            """
            The compass and the display are in the same orientation.
            """
            DEGREES_90: Config.DisplayConfig._CompassOrientation.ValueType  # 1
            """
            Rotate the compass by 90 degrees.
            """
            DEGREES_180: Config.DisplayConfig._CompassOrientation.ValueType  # 2
            """
            Rotate the compass by 180 degrees.
            """
            DEGREES_270: Config.DisplayConfig._CompassOrientation.ValueType  # 3
            """
            Rotate the compass by 270 degrees.
            """
            DEGREES_0_INVERTED: Config.DisplayConfig._CompassOrientation.ValueType  # 4
            """
            Don't rotate the compass, but invert the result.
            """
            DEGREES_90_INVERTED: Config.DisplayConfig._CompassOrientation.ValueType  # 5
            """
            Rotate the compass by 90 degrees and invert.
            """
            DEGREES_180_INVERTED: Config.DisplayConfig._CompassOrientation.ValueType  # 6
            """
            Rotate the compass by 180 degrees and invert.
            """
            DEGREES_270_INVERTED: Config.DisplayConfig._CompassOrientation.ValueType  # 7
            """
            Rotate the compass by 270 degrees and invert.
            """

        class CompassOrientation(_CompassOrientation, metaclass=_CompassOrientationEnumTypeWrapper): ...
        DEGREES_0: Config.DisplayConfig.CompassOrientation.ValueType  # 0
        """
        The compass and the display are in the same orientation.
        """
        DEGREES_90: Config.DisplayConfig.CompassOrientation.ValueType  # 1
        """
        Rotate the compass by 90 degrees.
        """
        DEGREES_180: Config.DisplayConfig.CompassOrientation.ValueType  # 2
        """
        Rotate the compass by 180 degrees.
        """
        DEGREES_270: Config.DisplayConfig.CompassOrientation.ValueType  # 3
        """
        Rotate the compass by 270 degrees.
        """
        DEGREES_0_INVERTED: Config.DisplayConfig.CompassOrientation.ValueType  # 4
        """
        Don't rotate the compass, but invert the result.
        """
        DEGREES_90_INVERTED: Config.DisplayConfig.CompassOrientation.ValueType  # 5
        """
        Rotate the compass by 90 degrees and invert.
        """
        DEGREES_180_INVERTED: Config.DisplayConfig.CompassOrientation.ValueType  # 6
        """
        Rotate the compass by 180 degrees and invert.
        """
        DEGREES_270_INVERTED: Config.DisplayConfig.CompassOrientation.ValueType  # 7
        """
        Rotate the compass by 270 degrees and invert.
        """

        SCREEN_ON_SECS_FIELD_NUMBER: builtins.int
        GPS_FORMAT_FIELD_NUMBER: builtins.int
        AUTO_SCREEN_CAROUSEL_SECS_FIELD_NUMBER: builtins.int
        COMPASS_NORTH_TOP_FIELD_NUMBER: builtins.int
        FLIP_SCREEN_FIELD_NUMBER: builtins.int
        UNITS_FIELD_NUMBER: builtins.int
        OLED_FIELD_NUMBER: builtins.int
        DISPLAYMODE_FIELD_NUMBER: builtins.int
        HEADING_BOLD_FIELD_NUMBER: builtins.int
        WAKE_ON_TAP_OR_MOTION_FIELD_NUMBER: builtins.int
        COMPASS_ORIENTATION_FIELD_NUMBER: builtins.int
        USE_12H_CLOCK_FIELD_NUMBER: builtins.int
        screen_on_secs: builtins.int
        """
        Number of seconds the screen stays on after pressing the user button or receiving a message
        0 for default of one minute MAXUINT for always on
        """
        gps_format: global___Config.DisplayConfig.GpsCoordinateFormat.ValueType
        """
        How the GPS coordinates are formatted on the OLED screen.
        """
        auto_screen_carousel_secs: builtins.int
        """
        Automatically toggles to the next page on the screen like a carousel, based the specified interval in seconds.
        Potentially useful for devices without user buttons.
        """
        compass_north_top: builtins.bool
        """
        If this is set, the displayed compass will always point north. if unset, the old behaviour
        (top of display is heading direction) is used.
        """
        flip_screen: builtins.bool
        """
        Flip screen vertically, for cases that mount the screen upside down
        """
        units: global___Config.DisplayConfig.DisplayUnits.ValueType
        """
        Perferred display units
        """
        oled: global___Config.DisplayConfig.OledType.ValueType
        """
        Override auto-detect in screen
        """
        displaymode: global___Config.DisplayConfig.DisplayMode.ValueType
        """
        Display Mode
        """
        heading_bold: builtins.bool
        """
        Print first line in pseudo-bold? FALSE is original style, TRUE is bold
        """
        wake_on_tap_or_motion: builtins.bool
        """
        Should we wake the screen up on accelerometer detected motion or tap
        """
        compass_orientation: global___Config.DisplayConfig.CompassOrientation.ValueType
        """
        Indicates how to rotate or invert the compass output to accurate display on the display.
        """
        use_12h_clock: builtins.bool
        """
        If false (default), the device will display the time in 24-hour format on screen.
        If true, the device will display the time in 12-hour format on screen.
        """
        def __init__(
            self,
            *,
            screen_on_secs: builtins.int = ...,
            gps_format: global___Config.DisplayConfig.GpsCoordinateFormat.ValueType = ...,
            auto_screen_carousel_secs: builtins.int = ...,
            compass_north_top: builtins.bool = ...,
            flip_screen: builtins.bool = ...,
            units: global___Config.DisplayConfig.DisplayUnits.ValueType = ...,
            oled: global___Config.DisplayConfig.OledType.ValueType = ...,
            displaymode: global___Config.DisplayConfig.DisplayMode.ValueType = ...,
            heading_bold: builtins.bool = ...,
            wake_on_tap_or_motion: builtins.bool = ...,
            compass_orientation: global___Config.DisplayConfig.CompassOrientation.ValueType = ...,
            use_12h_clock: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["auto_screen_carousel_secs", b"auto_screen_carousel_secs", "compass_north_top", b"compass_north_top", "compass_orientation", b"compass_orientation", "displaymode", b"displaymode", "flip_screen", b"flip_screen", "gps_format", b"gps_format", "heading_bold", b"heading_bold", "oled", b"oled", "screen_on_secs", b"screen_on_secs", "units", b"units", "use_12h_clock", b"use_12h_clock", "wake_on_tap_or_motion", b"wake_on_tap_or_motion"]) -> None: ...

    @typing.final
    class LoRaConfig(google.protobuf.message.Message):
        """
        Lora Config
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _RegionCode:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _RegionCodeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.LoRaConfig._RegionCode.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            UNSET: Config.LoRaConfig._RegionCode.ValueType  # 0
            """
            Region is not set
            """
            US: Config.LoRaConfig._RegionCode.ValueType  # 1
            """
            United States
            """
            EU_433: Config.LoRaConfig._RegionCode.ValueType  # 2
            """
            European Union 433mhz
            """
            EU_868: Config.LoRaConfig._RegionCode.ValueType  # 3
            """
            European Union 868mhz
            """
            CN: Config.LoRaConfig._RegionCode.ValueType  # 4
            """
            China
            """
            JP: Config.LoRaConfig._RegionCode.ValueType  # 5
            """
            Japan
            """
            ANZ: Config.LoRaConfig._RegionCode.ValueType  # 6
            """
            Australia / New Zealand
            """
            KR: Config.LoRaConfig._RegionCode.ValueType  # 7
            """
            Korea
            """
            TW: Config.LoRaConfig._RegionCode.ValueType  # 8
            """
            Taiwan
            """
            RU: Config.LoRaConfig._RegionCode.ValueType  # 9
            """
            Russia
            """
            IN: Config.LoRaConfig._RegionCode.ValueType  # 10
            """
            India
            """
            NZ_865: Config.LoRaConfig._RegionCode.ValueType  # 11
            """
            New Zealand 865mhz
            """
            TH: Config.LoRaConfig._RegionCode.ValueType  # 12
            """
            Thailand
            """
            LORA_24: Config.LoRaConfig._RegionCode.ValueType  # 13
            """
            WLAN Band
            """
            UA_433: Config.LoRaConfig._RegionCode.ValueType  # 14
            """
            Ukraine 433mhz
            """
            UA_868: Config.LoRaConfig._RegionCode.ValueType  # 15
            """
            Ukraine 868mhz
            """
            MY_433: Config.LoRaConfig._RegionCode.ValueType  # 16
            """
            Malaysia 433mhz
            """
            MY_919: Config.LoRaConfig._RegionCode.ValueType  # 17
            """
            Malaysia 919mhz
            """
            SG_923: Config.LoRaConfig._RegionCode.ValueType  # 18
            """
            Singapore 923mhz
            """
            PH_433: Config.LoRaConfig._RegionCode.ValueType  # 19
            """
            Philippines 433mhz
            """
            PH_868: Config.LoRaConfig._RegionCode.ValueType  # 20
            """
            Philippines 868mhz
            """
            PH_915: Config.LoRaConfig._RegionCode.ValueType  # 21
            """
            Philippines 915mhz
            """

        class RegionCode(_RegionCode, metaclass=_RegionCodeEnumTypeWrapper): ...
        UNSET: Config.LoRaConfig.RegionCode.ValueType  # 0
        """
        Region is not set
        """
        US: Config.LoRaConfig.RegionCode.ValueType  # 1
        """
        United States
        """
        EU_433: Config.LoRaConfig.RegionCode.ValueType  # 2
        """
        European Union 433mhz
        """
        EU_868: Config.LoRaConfig.RegionCode.ValueType  # 3
        """
        European Union 868mhz
        """
        CN: Config.LoRaConfig.RegionCode.ValueType  # 4
        """
        China
        """
        JP: Config.LoRaConfig.RegionCode.ValueType  # 5
        """
        Japan
        """
        ANZ: Config.LoRaConfig.RegionCode.ValueType  # 6
        """
        Australia / New Zealand
        """
        KR: Config.LoRaConfig.RegionCode.ValueType  # 7
        """
        Korea
        """
        TW: Config.LoRaConfig.RegionCode.ValueType  # 8
        """
        Taiwan
        """
        RU: Config.LoRaConfig.RegionCode.ValueType  # 9
        """
        Russia
        """
        IN: Config.LoRaConfig.RegionCode.ValueType  # 10
        """
        India
        """
        NZ_865: Config.LoRaConfig.RegionCode.ValueType  # 11
        """
        New Zealand 865mhz
        """
        TH: Config.LoRaConfig.RegionCode.ValueType  # 12
        """
        Thailand
        """
        LORA_24: Config.LoRaConfig.RegionCode.ValueType  # 13
        """
        WLAN Band
        """
        UA_433: Config.LoRaConfig.RegionCode.ValueType  # 14
        """
        Ukraine 433mhz
        """
        UA_868: Config.LoRaConfig.RegionCode.ValueType  # 15
        """
        Ukraine 868mhz
        """
        MY_433: Config.LoRaConfig.RegionCode.ValueType  # 16
        """
        Malaysia 433mhz
        """
        MY_919: Config.LoRaConfig.RegionCode.ValueType  # 17
        """
        Malaysia 919mhz
        """
        SG_923: Config.LoRaConfig.RegionCode.ValueType  # 18
        """
        Singapore 923mhz
        """
        PH_433: Config.LoRaConfig.RegionCode.ValueType  # 19
        """
        Philippines 433mhz
        """
        PH_868: Config.LoRaConfig.RegionCode.ValueType  # 20
        """
        Philippines 868mhz
        """
        PH_915: Config.LoRaConfig.RegionCode.ValueType  # 21
        """
        Philippines 915mhz
        """

        class _ModemPreset:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _ModemPresetEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.LoRaConfig._ModemPreset.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            LONG_FAST: Config.LoRaConfig._ModemPreset.ValueType  # 0
            """
            Long Range - Fast
            """
            LONG_SLOW: Config.LoRaConfig._ModemPreset.ValueType  # 1
            """
            Long Range - Slow
            """
            VERY_LONG_SLOW: Config.LoRaConfig._ModemPreset.ValueType  # 2
            """
            Very Long Range - Slow
            Deprecated in 2.5: Works only with txco and is unusably slow
            """
            MEDIUM_SLOW: Config.LoRaConfig._ModemPreset.ValueType  # 3
            """
            Medium Range - Slow
            """
            MEDIUM_FAST: Config.LoRaConfig._ModemPreset.ValueType  # 4
            """
            Medium Range - Fast
            """
            SHORT_SLOW: Config.LoRaConfig._ModemPreset.ValueType  # 5
            """
            Short Range - Slow
            """
            SHORT_FAST: Config.LoRaConfig._ModemPreset.ValueType  # 6
            """
            Short Range - Fast
            """
            LONG_MODERATE: Config.LoRaConfig._ModemPreset.ValueType  # 7
            """
            Long Range - Moderately Fast
            """
            SHORT_TURBO: Config.LoRaConfig._ModemPreset.ValueType  # 8
            """
            Short Range - Turbo
            This is the fastest preset and the only one with 500kHz bandwidth.
            It is not legal to use in all regions due to this wider bandwidth.
            """

        class ModemPreset(_ModemPreset, metaclass=_ModemPresetEnumTypeWrapper):
            """
            Standard predefined channel settings
            Note: these mappings must match ModemPreset Choice in the device code.
            """

        LONG_FAST: Config.LoRaConfig.ModemPreset.ValueType  # 0
        """
        Long Range - Fast
        """
        LONG_SLOW: Config.LoRaConfig.ModemPreset.ValueType  # 1
        """
        Long Range - Slow
        """
        VERY_LONG_SLOW: Config.LoRaConfig.ModemPreset.ValueType  # 2
        """
        Very Long Range - Slow
        Deprecated in 2.5: Works only with txco and is unusably slow
        """
        MEDIUM_SLOW: Config.LoRaConfig.ModemPreset.ValueType  # 3
        """
        Medium Range - Slow
        """
        MEDIUM_FAST: Config.LoRaConfig.ModemPreset.ValueType  # 4
        """
        Medium Range - Fast
        """
        SHORT_SLOW: Config.LoRaConfig.ModemPreset.ValueType  # 5
        """
        Short Range - Slow
        """
        SHORT_FAST: Config.LoRaConfig.ModemPreset.ValueType  # 6
        """
        Short Range - Fast
        """
        LONG_MODERATE: Config.LoRaConfig.ModemPreset.ValueType  # 7
        """
        Long Range - Moderately Fast
        """
        SHORT_TURBO: Config.LoRaConfig.ModemPreset.ValueType  # 8
        """
        Short Range - Turbo
        This is the fastest preset and the only one with 500kHz bandwidth.
        It is not legal to use in all regions due to this wider bandwidth.
        """

        USE_PRESET_FIELD_NUMBER: builtins.int
        MODEM_PRESET_FIELD_NUMBER: builtins.int
        BANDWIDTH_FIELD_NUMBER: builtins.int
        SPREAD_FACTOR_FIELD_NUMBER: builtins.int
        CODING_RATE_FIELD_NUMBER: builtins.int
        FREQUENCY_OFFSET_FIELD_NUMBER: builtins.int
        REGION_FIELD_NUMBER: builtins.int
        HOP_LIMIT_FIELD_NUMBER: builtins.int
        TX_ENABLED_FIELD_NUMBER: builtins.int
        TX_POWER_FIELD_NUMBER: builtins.int
        CHANNEL_NUM_FIELD_NUMBER: builtins.int
        OVERRIDE_DUTY_CYCLE_FIELD_NUMBER: builtins.int
        SX126X_RX_BOOSTED_GAIN_FIELD_NUMBER: builtins.int
        OVERRIDE_FREQUENCY_FIELD_NUMBER: builtins.int
        PA_FAN_DISABLED_FIELD_NUMBER: builtins.int
        IGNORE_INCOMING_FIELD_NUMBER: builtins.int
        IGNORE_MQTT_FIELD_NUMBER: builtins.int
        CONFIG_OK_TO_MQTT_FIELD_NUMBER: builtins.int
        use_preset: builtins.bool
        """
        When enabled, the `modem_preset` fields will be adhered to, else the `bandwidth`/`spread_factor`/`coding_rate`
        will be taked from their respective manually defined fields
        """
        modem_preset: global___Config.LoRaConfig.ModemPreset.ValueType
        """
        Either modem_config or bandwidth/spreading/coding will be specified - NOT BOTH.
        As a heuristic: If bandwidth is specified, do not use modem_config.
        Because protobufs take ZERO space when the value is zero this works out nicely.
        This value is replaced by bandwidth/spread_factor/coding_rate.
        If you'd like to experiment with other options add them to MeshRadio.cpp in the device code.
        """
        bandwidth: builtins.int
        """
        Bandwidth in MHz
        Certain bandwidth numbers are 'special' and will be converted to the
        appropriate floating point value: 31 -> 31.25MHz
        """
        spread_factor: builtins.int
        """
        A number from 7 to 12.
        Indicates number of chirps per symbol as 1<<spread_factor.
        """
        coding_rate: builtins.int
        """
        The denominator of the coding rate.
        ie for 4/5, the value is 5. 4/8 the value is 8.
        """
        frequency_offset: builtins.float
        """
        This parameter is for advanced users with advanced test equipment, we do not recommend most users use it.
        A frequency offset that is added to to the calculated band center frequency.
        Used to correct for crystal calibration errors.
        """
        region: global___Config.LoRaConfig.RegionCode.ValueType
        """
        The region code for the radio (US, CN, EU433, etc...)
        """
        hop_limit: builtins.int
        """
        Maximum number of hops. This can't be greater than 7.
        Default of 3
        Attempting to set a value > 7 results in the default
        """
        tx_enabled: builtins.bool
        """
        Disable TX from the LoRa radio. Useful for hot-swapping antennas and other tests.
        Defaults to false
        """
        tx_power: builtins.int
        """
        If zero, then use default max legal continuous power (ie. something that won't
        burn out the radio hardware)
        In most cases you should use zero here.
        Units are in dBm.
        """
        channel_num: builtins.int
        """
        This controls the actual hardware frequency the radio transmits on.
        Most users should never need to be exposed to this field/concept.
        A channel number between 1 and NUM_CHANNELS (whatever the max is in the current region).
        If ZERO then the rule is "use the old channel name hash based
        algorithm to derive the channel number")
        If using the hash algorithm the channel number will be: hash(channel_name) %
        NUM_CHANNELS (Where num channels depends on the regulatory region).
        """
        override_duty_cycle: builtins.bool
        """
        If true, duty cycle limits will be exceeded and thus you're possibly not following
        the local regulations if you're not a HAM.
        Has no effect if the duty cycle of the used region is 100%.
        """
        sx126x_rx_boosted_gain: builtins.bool
        """
        If true, sets RX boosted gain mode on SX126X based radios
        """
        override_frequency: builtins.float
        """
        This parameter is for advanced users and licensed HAM radio operators.
        Ignore Channel Calculation and use this frequency instead. The frequency_offset
        will still be applied. This will allow you to use out-of-band frequencies.
        Please respect your local laws and regulations. If you are a HAM, make sure you
        enable HAM mode and turn off encryption.
        """
        pa_fan_disabled: builtins.bool
        """
        If true, disable the build-in PA FAN using pin define in RF95_FAN_EN.
        """
        ignore_mqtt: builtins.bool
        """
        If true, the device will not process any packets received via LoRa that passed via MQTT anywhere on the path towards it.
        """
        config_ok_to_mqtt: builtins.bool
        """
        Sets the ok_to_mqtt bit on outgoing packets
        """
        @property
        def ignore_incoming(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.int]:
            """
            For testing it is useful sometimes to force a node to never listen to
            particular other nodes (simulating radio out of range). All nodenums listed
            in ignore_incoming will have packets they send dropped on receive (by router.cpp)
            """

        def __init__(
            self,
            *,
            use_preset: builtins.bool = ...,
            modem_preset: global___Config.LoRaConfig.ModemPreset.ValueType = ...,
            bandwidth: builtins.int = ...,
            spread_factor: builtins.int = ...,
            coding_rate: builtins.int = ...,
            frequency_offset: builtins.float = ...,
            region: global___Config.LoRaConfig.RegionCode.ValueType = ...,
            hop_limit: builtins.int = ...,
            tx_enabled: builtins.bool = ...,
            tx_power: builtins.int = ...,
            channel_num: builtins.int = ...,
            override_duty_cycle: builtins.bool = ...,
            sx126x_rx_boosted_gain: builtins.bool = ...,
            override_frequency: builtins.float = ...,
            pa_fan_disabled: builtins.bool = ...,
            ignore_incoming: collections.abc.Iterable[builtins.int] | None = ...,
            ignore_mqtt: builtins.bool = ...,
            config_ok_to_mqtt: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["bandwidth", b"bandwidth", "channel_num", b"channel_num", "coding_rate", b"coding_rate", "config_ok_to_mqtt", b"config_ok_to_mqtt", "frequency_offset", b"frequency_offset", "hop_limit", b"hop_limit", "ignore_incoming", b"ignore_incoming", "ignore_mqtt", b"ignore_mqtt", "modem_preset", b"modem_preset", "override_duty_cycle", b"override_duty_cycle", "override_frequency", b"override_frequency", "pa_fan_disabled", b"pa_fan_disabled", "region", b"region", "spread_factor", b"spread_factor", "sx126x_rx_boosted_gain", b"sx126x_rx_boosted_gain", "tx_enabled", b"tx_enabled", "tx_power", b"tx_power", "use_preset", b"use_preset"]) -> None: ...

    @typing.final
    class BluetoothConfig(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        class _PairingMode:
            ValueType = typing.NewType("ValueType", builtins.int)
            V: typing_extensions.TypeAlias = ValueType

        class _PairingModeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[Config.BluetoothConfig._PairingMode.ValueType], builtins.type):
            DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
            RANDOM_PIN: Config.BluetoothConfig._PairingMode.ValueType  # 0
            """
            Device generates a random PIN that will be shown on the screen of the device for pairing
            """
            FIXED_PIN: Config.BluetoothConfig._PairingMode.ValueType  # 1
            """
            Device requires a specified fixed PIN for pairing
            """
            NO_PIN: Config.BluetoothConfig._PairingMode.ValueType  # 2
            """
            Device requires no PIN for pairing
            """

        class PairingMode(_PairingMode, metaclass=_PairingModeEnumTypeWrapper): ...
        RANDOM_PIN: Config.BluetoothConfig.PairingMode.ValueType  # 0
        """
        Device generates a random PIN that will be shown on the screen of the device for pairing
        """
        FIXED_PIN: Config.BluetoothConfig.PairingMode.ValueType  # 1
        """
        Device requires a specified fixed PIN for pairing
        """
        NO_PIN: Config.BluetoothConfig.PairingMode.ValueType  # 2
        """
        Device requires no PIN for pairing
        """

        ENABLED_FIELD_NUMBER: builtins.int
        MODE_FIELD_NUMBER: builtins.int
        FIXED_PIN_FIELD_NUMBER: builtins.int
        enabled: builtins.bool
        """
        Enable Bluetooth on the device
        """
        mode: global___Config.BluetoothConfig.PairingMode.ValueType
        """
        Determines the pairing strategy for the device
        """
        fixed_pin: builtins.int
        """
        Specified PIN for PairingMode.FixedPin
        """
        def __init__(
            self,
            *,
            enabled: builtins.bool = ...,
            mode: global___Config.BluetoothConfig.PairingMode.ValueType = ...,
            fixed_pin: builtins.int = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["enabled", b"enabled", "fixed_pin", b"fixed_pin", "mode", b"mode"]) -> None: ...

    @typing.final
    class SecurityConfig(google.protobuf.message.Message):
        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        PUBLIC_KEY_FIELD_NUMBER: builtins.int
        PRIVATE_KEY_FIELD_NUMBER: builtins.int
        ADMIN_KEY_FIELD_NUMBER: builtins.int
        IS_MANAGED_FIELD_NUMBER: builtins.int
        SERIAL_ENABLED_FIELD_NUMBER: builtins.int
        DEBUG_LOG_API_ENABLED_FIELD_NUMBER: builtins.int
        ADMIN_CHANNEL_ENABLED_FIELD_NUMBER: builtins.int
        public_key: builtins.bytes
        """
        The public key of the user's device.
        Sent out to other nodes on the mesh to allow them to compute a shared secret key.
        """
        private_key: builtins.bytes
        """
        The private key of the device.
        Used to create a shared key with a remote device.
        """
        is_managed: builtins.bool
        """
        If true, device is considered to be "managed" by a mesh administrator via admin messages
        Device is managed by a mesh administrator.
        """
        serial_enabled: builtins.bool
        """
        Serial Console over the Stream API."
        """
        debug_log_api_enabled: builtins.bool
        """
        By default we turn off logging as soon as an API client connects (to keep shared serial link quiet).
        Output live debug logging over serial or bluetooth is set to true.
        """
        admin_channel_enabled: builtins.bool
        """
        Allow incoming device control over the insecure legacy admin channel.
        """
        @property
        def admin_key(self) -> google.protobuf.internal.containers.RepeatedScalarFieldContainer[builtins.bytes]:
            """
            The public key authorized to send admin messages to this node.
            """

        def __init__(
            self,
            *,
            public_key: builtins.bytes = ...,
            private_key: builtins.bytes = ...,
            admin_key: collections.abc.Iterable[builtins.bytes] | None = ...,
            is_managed: builtins.bool = ...,
            serial_enabled: builtins.bool = ...,
            debug_log_api_enabled: builtins.bool = ...,
            admin_channel_enabled: builtins.bool = ...,
        ) -> None: ...
        def ClearField(self, field_name: typing.Literal["admin_channel_enabled", b"admin_channel_enabled", "admin_key", b"admin_key", "debug_log_api_enabled", b"debug_log_api_enabled", "is_managed", b"is_managed", "private_key", b"private_key", "public_key", b"public_key", "serial_enabled", b"serial_enabled"]) -> None: ...

    @typing.final
    class SessionkeyConfig(google.protobuf.message.Message):
        """
        Blank config request, strictly for getting the session key
        """

        DESCRIPTOR: google.protobuf.descriptor.Descriptor

        def __init__(
            self,
        ) -> None: ...

    DEVICE_FIELD_NUMBER: builtins.int
    POSITION_FIELD_NUMBER: builtins.int
    POWER_FIELD_NUMBER: builtins.int
    NETWORK_FIELD_NUMBER: builtins.int
    DISPLAY_FIELD_NUMBER: builtins.int
    LORA_FIELD_NUMBER: builtins.int
    BLUETOOTH_FIELD_NUMBER: builtins.int
    SECURITY_FIELD_NUMBER: builtins.int
    SESSIONKEY_FIELD_NUMBER: builtins.int
    DEVICE_UI_FIELD_NUMBER: builtins.int
    @property
    def device(self) -> global___Config.DeviceConfig: ...
    @property
    def position(self) -> global___Config.PositionConfig: ...
    @property
    def power(self) -> global___Config.PowerConfig: ...
    @property
    def network(self) -> global___Config.NetworkConfig: ...
    @property
    def display(self) -> global___Config.DisplayConfig: ...
    @property
    def lora(self) -> global___Config.LoRaConfig: ...
    @property
    def bluetooth(self) -> global___Config.BluetoothConfig: ...
    @property
    def security(self) -> global___Config.SecurityConfig: ...
    @property
    def sessionkey(self) -> global___Config.SessionkeyConfig: ...
    @property
    def device_ui(self) -> meshtastic.protobuf.device_ui_pb2.DeviceUIConfig: ...
    def __init__(
        self,
        *,
        device: global___Config.DeviceConfig | None = ...,
        position: global___Config.PositionConfig | None = ...,
        power: global___Config.PowerConfig | None = ...,
        network: global___Config.NetworkConfig | None = ...,
        display: global___Config.DisplayConfig | None = ...,
        lora: global___Config.LoRaConfig | None = ...,
        bluetooth: global___Config.BluetoothConfig | None = ...,
        security: global___Config.SecurityConfig | None = ...,
        sessionkey: global___Config.SessionkeyConfig | None = ...,
        device_ui: meshtastic.protobuf.device_ui_pb2.DeviceUIConfig | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["bluetooth", b"bluetooth", "device", b"device", "device_ui", b"device_ui", "display", b"display", "lora", b"lora", "network", b"network", "payload_variant", b"payload_variant", "position", b"position", "power", b"power", "security", b"security", "sessionkey", b"sessionkey"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["bluetooth", b"bluetooth", "device", b"device", "device_ui", b"device_ui", "display", b"display", "lora", b"lora", "network", b"network", "payload_variant", b"payload_variant", "position", b"position", "power", b"power", "security", b"security", "sessionkey", b"sessionkey"]) -> None: ...
    def WhichOneof(self, oneof_group: typing.Literal["payload_variant", b"payload_variant"]) -> typing.Literal["device", "position", "power", "network", "display", "lora", "bluetooth", "security", "sessionkey", "device_ui"] | None: ...

global___Config = Config



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import collections.abc
import google.protobuf.descriptor
import google.protobuf.internal.containers
import google.protobuf.message
import meshtastic.protobuf.channel_pb2
import meshtastic.protobuf.config_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class ChannelSet(google.protobuf.message.Message):
    """
    This is the most compact possible representation for a set of channels.
    It includes only one PRIMARY channel (which must be first) and
    any SECONDARY channels.
    No DISABLED channels are included.
    This abstraction is used only on the the 'app side' of the world (ie python, javascript and android etc) to show a group of Channels as a (long) URL
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    SETTINGS_FIELD_NUMBER: builtins.int
    LORA_CONFIG_FIELD_NUMBER: builtins.int
    @property
    def settings(self) -> google.protobuf.internal.containers.RepeatedCompositeFieldContainer[meshtastic.protobuf.channel_pb2.ChannelSettings]:
        """
        Channel list with settings
        """

    @property
    def lora_config(self) -> meshtastic.protobuf.config_pb2.Config.LoRaConfig:
        """
        LoRa config
        """

    def __init__(
        self,
        *,
        settings: collections.abc.Iterable[meshtastic.protobuf.channel_pb2.ChannelSettings] | None = ...,
        lora_config: meshtastic.protobuf.config_pb2.Config.LoRaConfig | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["lora_config", b"lora_config"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["lora_config", b"lora_config", "settings", b"settings"]) -> None: ...

global___ChannelSet = ChannelSet



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.internal.enum_type_wrapper
import google.protobuf.message
import sys
import typing

if sys.version_info >= (3, 10):
    import typing as typing_extensions
else:
    import typing_extensions

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

class _Theme:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _ThemeEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Theme.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    DARK: _Theme.ValueType  # 0
    """
    Dark
    """
    LIGHT: _Theme.ValueType  # 1
    """
    Light
    """
    RED: _Theme.ValueType  # 2
    """
    Red
    """

class Theme(_Theme, metaclass=_ThemeEnumTypeWrapper): ...

DARK: Theme.ValueType  # 0
"""
Dark
"""
LIGHT: Theme.ValueType  # 1
"""
Light
"""
RED: Theme.ValueType  # 2
"""
Red
"""
global___Theme = Theme

class _Language:
    ValueType = typing.NewType("ValueType", builtins.int)
    V: typing_extensions.TypeAlias = ValueType

class _LanguageEnumTypeWrapper(google.protobuf.internal.enum_type_wrapper._EnumTypeWrapper[_Language.ValueType], builtins.type):
    DESCRIPTOR: google.protobuf.descriptor.EnumDescriptor
    ENGLISH: _Language.ValueType  # 0
    """
    English
    """
    FRENCH: _Language.ValueType  # 1
    """
    French
    """
    GERMAN: _Language.ValueType  # 2
    """
    German
    """
    ITALIAN: _Language.ValueType  # 3
    """
    Italian
    """
    PORTUGUESE: _Language.ValueType  # 4
    """
    Portuguese
    """
    SPANISH: _Language.ValueType  # 5
    """
    Spanish
    """
    SWEDISH: _Language.ValueType  # 6
    """
    Swedish
    """
    FINNISH: _Language.ValueType  # 7
    """
    Finnish
    """
    POLISH: _Language.ValueType  # 8
    """
    Polish
    """
    TURKISH: _Language.ValueType  # 9
    """
    Turkish
    """
    SERBIAN: _Language.ValueType  # 10
    """
    Serbian
    """
    RUSSIAN: _Language.ValueType  # 11
    """
    Russian
    """
    DUTCH: _Language.ValueType  # 12
    """
    Dutch
    """
    GREEK: _Language.ValueType  # 13
    """
    Greek
    """
    NORWEGIAN: _Language.ValueType  # 14
    """
    Norwegian
    """
    SLOVENIAN: _Language.ValueType  # 15
    """
    Slovenian
    """
    SIMPLIFIED_CHINESE: _Language.ValueType  # 30
    """
    Simplified Chinese (experimental)
    """
    TRADITIONAL_CHINESE: _Language.ValueType  # 31
    """
    Traditional Chinese (experimental)
    """

class Language(_Language, metaclass=_LanguageEnumTypeWrapper):
    """
    Localization
    """

ENGLISH: Language.ValueType  # 0
"""
English
"""
FRENCH: Language.ValueType  # 1
"""
French
"""
GERMAN: Language.ValueType  # 2
"""
German
"""
ITALIAN: Language.ValueType  # 3
"""
Italian
"""
PORTUGUESE: Language.ValueType  # 4
"""
Portuguese
"""
SPANISH: Language.ValueType  # 5
"""
Spanish
"""
SWEDISH: Language.ValueType  # 6
"""
Swedish
"""
FINNISH: Language.ValueType  # 7
"""
Finnish
"""
POLISH: Language.ValueType  # 8
"""
Polish
"""
TURKISH: Language.ValueType  # 9
"""
Turkish
"""
SERBIAN: Language.ValueType  # 10
"""
Serbian
"""
RUSSIAN: Language.ValueType  # 11
"""
Russian
"""
DUTCH: Language.ValueType  # 12
"""
Dutch
"""
GREEK: Language.ValueType  # 13
"""
Greek
"""
NORWEGIAN: Language.ValueType  # 14
"""
Norwegian
"""
SLOVENIAN: Language.ValueType  # 15
"""
Slovenian
"""
SIMPLIFIED_CHINESE: Language.ValueType  # 30
"""
Simplified Chinese (experimental)
"""
TRADITIONAL_CHINESE: Language.ValueType  # 31
"""
Traditional Chinese (experimental)
"""
global___Language = Language

@typing.final
class DeviceUIConfig(google.protobuf.message.Message):
    """
    Protobuf structures for device-ui persistency
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    VERSION_FIELD_NUMBER: builtins.int
    SCREEN_BRIGHTNESS_FIELD_NUMBER: builtins.int
    SCREEN_TIMEOUT_FIELD_NUMBER: builtins.int
    SCREEN_LOCK_FIELD_NUMBER: builtins.int
    SETTINGS_LOCK_FIELD_NUMBER: builtins.int
    PIN_CODE_FIELD_NUMBER: builtins.int
    THEME_FIELD_NUMBER: builtins.int
    ALERT_ENABLED_FIELD_NUMBER: builtins.int
    BANNER_ENABLED_FIELD_NUMBER: builtins.int
    RING_TONE_ID_FIELD_NUMBER: builtins.int
    LANGUAGE_FIELD_NUMBER: builtins.int
    NODE_FILTER_FIELD_NUMBER: builtins.int
    NODE_HIGHLIGHT_FIELD_NUMBER: builtins.int
    CALIBRATION_DATA_FIELD_NUMBER: builtins.int
    MAP_DATA_FIELD_NUMBER: builtins.int
    version: builtins.int
    """
    A version integer used to invalidate saved files when we make incompatible changes.
    """
    screen_brightness: builtins.int
    """
    TFT display brightness 1..255
    """
    screen_timeout: builtins.int
    """
    Screen timeout 0..900
    """
    screen_lock: builtins.bool
    """
    Screen/Settings lock enabled
    """
    settings_lock: builtins.bool
    pin_code: builtins.int
    theme: global___Theme.ValueType
    """
    Color theme
    """
    alert_enabled: builtins.bool
    """
    Audible message, banner and ring tone
    """
    banner_enabled: builtins.bool
    ring_tone_id: builtins.int
    language: global___Language.ValueType
    """
    Localization
    """
    calibration_data: builtins.bytes
    """
    8 integers for screen calibration data
    """
    @property
    def node_filter(self) -> global___NodeFilter:
        """
        Node list filter
        """

    @property
    def node_highlight(self) -> global___NodeHighlight:
        """
        Node list highlightening
        """

    @property
    def map_data(self) -> global___Map:
        """
        Map related data
        """

    def __init__(
        self,
        *,
        version: builtins.int = ...,
        screen_brightness: builtins.int = ...,
        screen_timeout: builtins.int = ...,
        screen_lock: builtins.bool = ...,
        settings_lock: builtins.bool = ...,
        pin_code: builtins.int = ...,
        theme: global___Theme.ValueType = ...,
        alert_enabled: builtins.bool = ...,
        banner_enabled: builtins.bool = ...,
        ring_tone_id: builtins.int = ...,
        language: global___Language.ValueType = ...,
        node_filter: global___NodeFilter | None = ...,
        node_highlight: global___NodeHighlight | None = ...,
        calibration_data: builtins.bytes = ...,
        map_data: global___Map | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["map_data", b"map_data", "node_filter", b"node_filter", "node_highlight", b"node_highlight"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["alert_enabled", b"alert_enabled", "banner_enabled", b"banner_enabled", "calibration_data", b"calibration_data", "language", b"language", "map_data", b"map_data", "node_filter", b"node_filter", "node_highlight", b"node_highlight", "pin_code", b"pin_code", "ring_tone_id", b"ring_tone_id", "screen_brightness", b"screen_brightness", "screen_lock", b"screen_lock", "screen_timeout", b"screen_timeout", "settings_lock", b"settings_lock", "theme", b"theme", "version", b"version"]) -> None: ...

global___DeviceUIConfig = DeviceUIConfig

@typing.final
class NodeFilter(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    UNKNOWN_SWITCH_FIELD_NUMBER: builtins.int
    OFFLINE_SWITCH_FIELD_NUMBER: builtins.int
    PUBLIC_KEY_SWITCH_FIELD_NUMBER: builtins.int
    HOPS_AWAY_FIELD_NUMBER: builtins.int
    POSITION_SWITCH_FIELD_NUMBER: builtins.int
    NODE_NAME_FIELD_NUMBER: builtins.int
    CHANNEL_FIELD_NUMBER: builtins.int
    unknown_switch: builtins.bool
    """
    Filter unknown nodes
    """
    offline_switch: builtins.bool
    """
    Filter offline nodes
    """
    public_key_switch: builtins.bool
    """
    Filter nodes w/o public key
    """
    hops_away: builtins.int
    """
    Filter based on hops away
    """
    position_switch: builtins.bool
    """
    Filter nodes w/o position
    """
    node_name: builtins.str
    """
    Filter nodes by matching name string
    """
    channel: builtins.int
    """
    Filter based on channel
    """
    def __init__(
        self,
        *,
        unknown_switch: builtins.bool = ...,
        offline_switch: builtins.bool = ...,
        public_key_switch: builtins.bool = ...,
        hops_away: builtins.int = ...,
        position_switch: builtins.bool = ...,
        node_name: builtins.str = ...,
        channel: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["channel", b"channel", "hops_away", b"hops_away", "node_name", b"node_name", "offline_switch", b"offline_switch", "position_switch", b"position_switch", "public_key_switch", b"public_key_switch", "unknown_switch", b"unknown_switch"]) -> None: ...

global___NodeFilter = NodeFilter

@typing.final
class NodeHighlight(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    CHAT_SWITCH_FIELD_NUMBER: builtins.int
    POSITION_SWITCH_FIELD_NUMBER: builtins.int
    TELEMETRY_SWITCH_FIELD_NUMBER: builtins.int
    IAQ_SWITCH_FIELD_NUMBER: builtins.int
    NODE_NAME_FIELD_NUMBER: builtins.int
    chat_switch: builtins.bool
    """
    Hightlight nodes w/ active chat
    """
    position_switch: builtins.bool
    """
    Highlight nodes w/ position
    """
    telemetry_switch: builtins.bool
    """
    Highlight nodes w/ telemetry data
    """
    iaq_switch: builtins.bool
    """
    Highlight nodes w/ iaq data
    """
    node_name: builtins.str
    """
    Highlight nodes by matching name string
    """
    def __init__(
        self,
        *,
        chat_switch: builtins.bool = ...,
        position_switch: builtins.bool = ...,
        telemetry_switch: builtins.bool = ...,
        iaq_switch: builtins.bool = ...,
        node_name: builtins.str = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["chat_switch", b"chat_switch", "iaq_switch", b"iaq_switch", "node_name", b"node_name", "position_switch", b"position_switch", "telemetry_switch", b"telemetry_switch"]) -> None: ...

global___NodeHighlight = NodeHighlight

@typing.final
class GeoPoint(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    ZOOM_FIELD_NUMBER: builtins.int
    LATITUDE_FIELD_NUMBER: builtins.int
    LONGITUDE_FIELD_NUMBER: builtins.int
    zoom: builtins.int
    """
    Zoom level
    """
    latitude: builtins.int
    """
    Coordinate: latitude
    """
    longitude: builtins.int
    """
    Coordinate: longitude
    """
    def __init__(
        self,
        *,
        zoom: builtins.int = ...,
        latitude: builtins.int = ...,
        longitude: builtins.int = ...,
    ) -> None: ...
    def ClearField(self, field_name: typing.Literal["latitude", b"latitude", "longitude", b"longitude", "zoom", b"zoom"]) -> None: ...

global___GeoPoint = GeoPoint

@typing.final
class Map(google.protobuf.message.Message):
    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    HOME_FIELD_NUMBER: builtins.int
    STYLE_FIELD_NUMBER: builtins.int
    FOLLOW_GPS_FIELD_NUMBER: builtins.int
    style: builtins.str
    """
    Map tile style
    """
    follow_gps: builtins.bool
    """
    Map scroll follows GPS
    """
    @property
    def home(self) -> global___GeoPoint:
        """
        Home coordinates
        """

    def __init__(
        self,
        *,
        home: global___GeoPoint | None = ...,
        style: builtins.str = ...,
        follow_gps: builtins.bool = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["home", b"home"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["follow_gps", b"follow_gps", "home", b"home", "style", b"style"]) -> None: ...

global___Map = Map



"""code logging power consumption of meshtastic devices."""

import logging
from datetime import datetime

from riden import Riden

from .power_supply import PowerSupply


class RidenPowerSupply(PowerSupply):
    """Interface for talking to Riden programmable bench-top power supplies.
    Only RD6006 tested but others should be similar.
    """

    def __init__(self, portName: str = "/dev/ttyUSB0"):
        """Initialize the RidenPowerSupply object.

        portName (str, optional): The port name of the power supply. Defaults to "/dev/ttyUSB0".
        """
        self.r = r = Riden(port=portName, baudrate=115200, address=1)
        logging.info(
            f"Connected to Riden power supply: model {r.type}, sn {r.sn}, firmware {r.fw}. Date/time updated."
        )
        r.set_date_time(datetime.now())
        self.prevWattHour = self._getRawWattHour()
        self.nowWattHour = self.prevWattHour
        super().__init__()  # we call this late so that the port is already open and _getRawWattHour callback works

    def setMaxCurrent(self, i: float):
        """Set the maximum current the supply will provide."""
        self.r.set_i_set(i)

    def powerOn(self):
        """Power on the supply, with reasonable defaults for meshtastic devices."""
        self.r.set_v_set(
            self.v
        )  # my WM1110 devboard header is directly connected to the 3.3V rail
        self.r.set_output(1)

    def get_average_current_mA(self) -> float:
        """Returns average current of last measurement in mA (since last call to this method)"""
        now = datetime.now()
        nowWattHour = self._getRawWattHour()
        watts = (
            (nowWattHour - self.prevWattHour)
            / (now - self.prevPowerTime).total_seconds()
            * 3600
        )
        self.prevPowerTime = now
        self.prevWattHour = nowWattHour
        return watts / 1000

    def _getRawWattHour(self) -> float:
        """Get the current watt-hour reading."""
        self.r.update()
        return self.r.wh



"""Remote hardware
"""
import logging

from pubsub import pub # type: ignore[import-untyped]

from meshtastic.protobuf import portnums_pb2, remote_hardware_pb2
from meshtastic.util import our_exit


def onGPIOreceive(packet, interface) -> None:
    """Callback for received GPIO responses"""
    logging.debug(f"packet:{packet} interface:{interface}")
    gpioValue = 0
    hw = packet["decoded"]["remotehw"]
    if "gpioValue" in hw:
        gpioValue = hw["gpioValue"]
    else:
        if not "gpioMask" in hw:
            # we did get a reply, but due to protobufs, 0 for numeric value is not sent
            # see https://developers.google.com/protocol-buffers/docs/proto3#default
            # so, we set it here
            gpioValue = 0

    # print(f'mask:{interface.mask}')
    value = int(gpioValue) & int(interface.mask)
    print(
        f'Received RemoteHardware type={hw["type"]}, gpio_value={gpioValue} value={value}'
    )
    interface.gotResponse = True


class RemoteHardwareClient:
    """
    This is the client code to control/monitor simple hardware built into the
    meshtastic devices.  It is intended to be both a useful API/service and example
    code for how you can connect to your own custom meshtastic services
    """

    def __init__(self, iface) -> None:
        """
        Constructor

        iface is the already open MeshInterface instance
        """
        self.iface = iface
        ch = iface.localNode.getChannelByName("gpio")
        if not ch:
            our_exit(
                "Warning: No channel named 'gpio' was found.\n"
                "On the sending and receive nodes create a channel named 'gpio'.\n"
                "For example, run '--ch-add gpio' on one device, then '--seturl' on\n"
                "the other devices using the url from the device where the channel was added."
            )
        self.channelIndex = ch.index

        pub.subscribe(onGPIOreceive, "meshtastic.receive.remotehw")

    def _sendHardware(self, nodeid, r, wantResponse=False, onResponse=None):
        if not nodeid:
            our_exit(
                r"Warning: Must use a destination node ID for this operation (use --dest \!xxxxxxxxx)"
            )
        return self.iface.sendData(
            r,
            nodeid,
            portnums_pb2.REMOTE_HARDWARE_APP,
            wantAck=True,
            channelIndex=self.channelIndex,
            wantResponse=wantResponse,
            onResponse=onResponse,
        )

    def writeGPIOs(self, nodeid, mask, vals):
        """
        Write the specified vals bits to the device GPIOs.  Only bits in mask that
        are 1 will be changed
        """
        logging.debug(f"writeGPIOs nodeid:{nodeid} mask:{mask} vals:{vals}")
        r = remote_hardware_pb2.HardwareMessage()
        r.type = remote_hardware_pb2.HardwareMessage.Type.WRITE_GPIOS
        r.gpio_mask = mask
        r.gpio_value = vals
        return self._sendHardware(nodeid, r)

    def readGPIOs(self, nodeid, mask, onResponse=None):
        """Read the specified bits from GPIO inputs on the device"""
        logging.debug(f"readGPIOs nodeid:{nodeid} mask:{mask}")
        r = remote_hardware_pb2.HardwareMessage()
        r.type = remote_hardware_pb2.HardwareMessage.Type.READ_GPIOS
        r.gpio_mask = mask
        return self._sendHardware(nodeid, r, wantResponse=True, onResponse=onResponse)

    def watchGPIOs(self, nodeid, mask):
        """Watch the specified bits from GPIO inputs on the device for changes"""
        logging.debug(f"watchGPIOs nodeid:{nodeid} mask:{mask}")
        r = remote_hardware_pb2.HardwareMessage()
        r.type = remote_hardware_pb2.HardwareMessage.Type.WATCH_GPIOS
        r.gpio_mask = mask
        self.iface.mask = mask
        return self._sendHardware(nodeid, r)



"""Meshtastic smoke tests with 2 devices connected via USB"""
import re
import subprocess

import pytest


@pytest.mark.smoke2
def test_smoke2_info():
    """Test --info with 2 devices connected serially"""
    return_value, out = subprocess.getstatusoutput("meshtastic --info")
    assert re.search(r"Warning: Multiple", out, re.MULTILINE)
    assert return_value == 1


@pytest.mark.smoke2
def test_smoke2_test():
    """Test --test"""
    return_value, out = subprocess.getstatusoutput("meshtastic --test")
    assert re.search(r"Writing serial debugging", out, re.MULTILINE)
    assert re.search(r"Ports opened", out, re.MULTILINE)
    assert re.search(r"Running 5 tests", out, re.MULTILINE)
    assert return_value == 0



"""
@generated by mypy-protobuf.  Do not edit manually!
isort:skip_file
"""

import builtins
import google.protobuf.descriptor
import google.protobuf.message
import meshtastic.protobuf.localonly_pb2
import meshtastic.protobuf.mesh_pb2
import typing

DESCRIPTOR: google.protobuf.descriptor.FileDescriptor

@typing.final
class DeviceProfile(google.protobuf.message.Message):
    """
    This abstraction is used to contain any configuration for provisioning a node on any client.
    It is useful for importing and exporting configurations.
    """

    DESCRIPTOR: google.protobuf.descriptor.Descriptor

    LONG_NAME_FIELD_NUMBER: builtins.int
    SHORT_NAME_FIELD_NUMBER: builtins.int
    CHANNEL_URL_FIELD_NUMBER: builtins.int
    CONFIG_FIELD_NUMBER: builtins.int
    MODULE_CONFIG_FIELD_NUMBER: builtins.int
    FIXED_POSITION_FIELD_NUMBER: builtins.int
    RINGTONE_FIELD_NUMBER: builtins.int
    CANNED_MESSAGES_FIELD_NUMBER: builtins.int
    long_name: builtins.str
    """
    Long name for the node
    """
    short_name: builtins.str
    """
    Short name of the node
    """
    channel_url: builtins.str
    """
    The url of the channels from our node
    """
    ringtone: builtins.str
    """
    Ringtone for ExternalNotification
    """
    canned_messages: builtins.str
    """
    Predefined messages for CannedMessage
    """
    @property
    def config(self) -> meshtastic.protobuf.localonly_pb2.LocalConfig:
        """
        The Config of the node
        """

    @property
    def module_config(self) -> meshtastic.protobuf.localonly_pb2.LocalModuleConfig:
        """
        The ModuleConfig of the node
        """

    @property
    def fixed_position(self) -> meshtastic.protobuf.mesh_pb2.Position:
        """
        Fixed position data
        """

    def __init__(
        self,
        *,
        long_name: builtins.str | None = ...,
        short_name: builtins.str | None = ...,
        channel_url: builtins.str | None = ...,
        config: meshtastic.protobuf.localonly_pb2.LocalConfig | None = ...,
        module_config: meshtastic.protobuf.localonly_pb2.LocalModuleConfig | None = ...,
        fixed_position: meshtastic.protobuf.mesh_pb2.Position | None = ...,
        ringtone: builtins.str | None = ...,
        canned_messages: builtins.str | None = ...,
    ) -> None: ...
    def HasField(self, field_name: typing.Literal["_canned_messages", b"_canned_messages", "_channel_url", b"_channel_url", "_config", b"_config", "_fixed_position", b"_fixed_position", "_long_name", b"_long_name", "_module_config", b"_module_config", "_ringtone", b"_ringtone", "_short_name", b"_short_name", "canned_messages", b"canned_messages", "channel_url", b"channel_url", "config", b"config", "fixed_position", b"fixed_position", "long_name", b"long_name", "module_config", b"module_config", "ringtone", b"ringtone", "short_name", b"short_name"]) -> builtins.bool: ...
    def ClearField(self, field_name: typing.Literal["_canned_messages", b"_canned_messages", "_channel_url", b"_channel_url", "_config", b"_config", "_fixed_position", b"_fixed_position", "_long_name", b"_long_name", "_module_config", b"_module_config", "_ringtone", b"_ringtone", "_short_name", b"_short_name", "canned_messages", b"canned_messages", "channel_url", b"channel_url", "config", b"config", "fixed_position", b"fixed_position", "long_name", b"long_name", "module_config", b"module_config", "ringtone", b"ringtone", "short_name", b"short_name"]) -> None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_canned_messages", b"_canned_messages"]) -> typing.Literal["canned_messages"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_channel_url", b"_channel_url"]) -> typing.Literal["channel_url"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_config", b"_config"]) -> typing.Literal["config"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_fixed_position", b"_fixed_position"]) -> typing.Literal["fixed_position"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_long_name", b"_long_name"]) -> typing.Literal["long_name"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_module_config", b"_module_config"]) -> typing.Literal["module_config"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_ringtone", b"_ringtone"]) -> typing.Literal["ringtone"] | None: ...
    @typing.overload
    def WhichOneof(self, oneof_group: typing.Literal["_short_name", b"_short_name"]) -> typing.Literal["short_name"] | None: ...

global___DeviceProfile = DeviceProfile



"""Utilities for Apache Arrow serialization."""

import logging
import threading
import os
from typing import Optional, List

import pyarrow as pa
from pyarrow import feather

chunk_size = 1000  # disk writes are batched based on this number of rows


class ArrowWriter:
    """Writes an arrow file in a streaming fashion"""

    def __init__(self, file_name: str):
        """Create a new ArrowWriter object.

        file_name (str): The name of the file to write to.
        """
        self.sink = pa.OSFile(file_name, "wb")  # type: ignore
        self.new_rows: List[dict] = []
        self.schema: Optional[pa.Schema] = None  # haven't yet learned the schema
        self.writer: Optional[pa.RecordBatchStreamWriter] = None
        self._lock = threading.Condition()  # Ensure only one thread writes at a time

    def close(self):
        """Close the stream and writes the file as needed."""
        with self._lock:
            self._write()
            if self.writer:
                self.writer.close()
            self.sink.close()

    def set_schema(self, schema: pa.Schema):
        """Set the schema for the file.
        Only needed for datasets where we can't learn it from the first record written.

        schema (pa.Schema): The schema to use.
        """
        with self._lock:
            assert self.schema is None
            self.schema = schema
            self.writer = pa.ipc.new_stream(self.sink, schema)

    def _write(self):
        """Write the new rows to the file."""
        if len(self.new_rows) > 0:
            if self.schema is None:
                # only need to look at the first row to learn the schema
                self.set_schema(pa.Table.from_pylist([self.new_rows[0]]).schema)

            self.writer.write_batch(
                pa.RecordBatch.from_pylist(self.new_rows, schema=self.schema)
            )
            self.new_rows = []

    def add_row(self, row_dict: dict):
        """Add a row to the arrow file.
        We will automatically learn the schema from the first row. But all rows must use that schema.
        """
        with self._lock:
            self.new_rows.append(row_dict)
            if len(self.new_rows) >= chunk_size:
                self._write()


class FeatherWriter(ArrowWriter):
    """A smaller more interoperable version of arrow files.
    Uses a temporary .arrow file (which could be huge) but converts to a much smaller (but still fast)
    feather file.
    """

    def __init__(self, file_name: str):
        super().__init__(file_name + ".arrow")
        self.base_file_name = file_name

    def close(self):
        super().close()
        src_name = self.base_file_name + ".arrow"
        dest_name = self.base_file_name + ".feather"
        if os.path.getsize(src_name) == 0:
            logging.warning(f"Discarding empty file: {src_name}")
            os.remove(src_name)
        else:
            logging.info(f"Compressing log data into {dest_name}")

            # note: must use open_stream, not open_file/read_table because the streaming layout is different
            # data = feather.read_table(src_name)
            with pa.memory_map(src_name) as source:
                array = pa.ipc.open_stream(source).read_all()

            # See https://stackoverflow.com/a/72406099 for more info and performance testing measurements
            feather.write_feather(array, dest_name, compression="zstd")
            os.remove(src_name)



"""Power stress testing support.
"""
import logging
import time

from ..protobuf import portnums_pb2, powermon_pb2


def onPowerStressResponse(packet, interface):
    """Delete me? FIXME"""
    logging.debug(f"packet:{packet} interface:{interface}")
    # interface.gotResponse = True


class PowerStressClient:
    """
    The client stub for talking to the firmware PowerStress module.
    """

    def __init__(self, iface, node_id=None):
        """
        Create a new PowerStressClient instance.

        iface is the already open MeshInterface instance
        """
        self.iface = iface

        if not node_id:
            node_id = iface.myInfo.my_node_num

        self.node_id = node_id
        # No need to subscribe - because we
        # pub.subscribe(onGPIOreceive, "meshtastic.receive.powerstress")

    def sendPowerStress(
        self,
        cmd: powermon_pb2.PowerStressMessage.Opcode.ValueType,
        num_seconds: float = 0.0,
        onResponse=None,
    ):
        """Client goo for talking with the device side agent."""
        r = powermon_pb2.PowerStressMessage()
        r.cmd = cmd
        r.num_seconds = num_seconds

        return self.iface.sendData(
            r,
            self.node_id,
            portnums_pb2.POWERSTRESS_APP,
            wantAck=True,
            wantResponse=True,
            onResponse=onResponse,
            onResponseAckPermitted=True,
        )

    def syncPowerStress(
        self,
        cmd: powermon_pb2.PowerStressMessage.Opcode.ValueType,
        num_seconds: float = 0.0,
    ):
        """Send a power stress command and wait for the ack."""
        gotAck = False

        def onResponse(packet: dict):  # pylint: disable=unused-argument
            nonlocal gotAck
            gotAck = True

        logging.info(
            f"Sending power stress command {powermon_pb2.PowerStressMessage.Opcode.Name(cmd)}"
        )
        self.sendPowerStress(cmd, onResponse=onResponse, num_seconds=num_seconds)

        if num_seconds == 0.0:
            # Wait for the response and then continue
            while not gotAck:
                time.sleep(0.1)
        else:
            # we wait a little bit longer than the time the UUT would be waiting (to make sure all of its messages are handled first)
            time.sleep(
                num_seconds + 0.2
            )  # completely block our thread for the duration of the test
            if not gotAck:
                logging.error("Did not receive ack for power stress command!")


class PowerStress:
    """Walk the UUT through a set of power states so we can capture repeatable power consumption measurements."""

    def __init__(self, iface):
        self.client = PowerStressClient(iface)

    def run(self):
        """Run the power stress test."""
        try:
            self.client.syncPowerStress(powermon_pb2.PowerStressMessage.PRINT_INFO)

            num_seconds = 5.0
            states = [
                powermon_pb2.PowerStressMessage.LED_ON,
                powermon_pb2.PowerStressMessage.LED_OFF,
                powermon_pb2.PowerStressMessage.BT_OFF,
                powermon_pb2.PowerStressMessage.BT_ON,
                powermon_pb2.PowerStressMessage.CPU_FULLON,
                powermon_pb2.PowerStressMessage.CPU_IDLE,
                # FIXME - can't test deepsleep yet because the ttyACM device disappears.  Fix the python code to retry connections
                # powermon_pb2.PowerStressMessage.CPU_DEEPSLEEP,
            ]
            for s in states:
                s_name = powermon_pb2.PowerStressMessage.Opcode.Name(s)
                logging.info(
                    f"Running power stress test {s_name} for {num_seconds} seconds"
                )
                self.client.syncPowerStress(s, num_seconds)

            logging.info("Power stress test complete.")
        except KeyboardInterrupt as e:
            logging.warning(f"Power stress interrupted: {e}")



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: meshtastic/protobuf/clientonly.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from meshtastic.protobuf import localonly_pb2 as meshtastic_dot_protobuf_dot_localonly__pb2
from meshtastic.protobuf import mesh_pb2 as meshtastic_dot_protobuf_dot_mesh__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n$meshtastic/protobuf/clientonly.proto\x12\x13meshtastic.protobuf\x1a#meshtastic/protobuf/localonly.proto\x1a\x1emeshtastic/protobuf/mesh.proto\"\xc4\x03\n\rDeviceProfile\x12\x16\n\tlong_name\x18\x01 \x01(\tH\x00\x88\x01\x01\x12\x17\n\nshort_name\x18\x02 \x01(\tH\x01\x88\x01\x01\x12\x18\n\x0b\x63hannel_url\x18\x03 \x01(\tH\x02\x88\x01\x01\x12\x35\n\x06\x63onfig\x18\x04 \x01(\x0b\x32 .meshtastic.protobuf.LocalConfigH\x03\x88\x01\x01\x12\x42\n\rmodule_config\x18\x05 \x01(\x0b\x32&.meshtastic.protobuf.LocalModuleConfigH\x04\x88\x01\x01\x12:\n\x0e\x66ixed_position\x18\x06 \x01(\x0b\x32\x1d.meshtastic.protobuf.PositionH\x05\x88\x01\x01\x12\x15\n\x08ringtone\x18\x07 \x01(\tH\x06\x88\x01\x01\x12\x1c\n\x0f\x63\x61nned_messages\x18\x08 \x01(\tH\x07\x88\x01\x01\x42\x0c\n\n_long_nameB\r\n\x0b_short_nameB\x0e\n\x0c_channel_urlB\t\n\x07_configB\x10\n\x0e_module_configB\x11\n\x0f_fixed_positionB\x0b\n\t_ringtoneB\x12\n\x10_canned_messagesBe\n\x13\x63om.geeksville.meshB\x10\x43lientOnlyProtosZ\"github.com/meshtastic/go/generated\xaa\x02\x14Meshtastic.Protobufs\xba\x02\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtastic.protobuf.clientonly_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.geeksville.meshB\020ClientOnlyProtosZ\"github.com/meshtastic/go/generated\252\002\024Meshtastic.Protobufs\272\002\000'
  _globals['_DEVICEPROFILE']._serialized_start=131
  _globals['_DEVICEPROFILE']._serialized_end=583
# @@protoc_insertion_point(module_scope)



"""Code for IP tunnel over a mesh

# Note python-pytuntap was too buggy
# using pip3 install pytap2
# make sure to "sudo setcap cap_net_admin+eip /usr/bin/python3.8" so python can access tun device without being root
# sudo ip tuntap del mode tun tun0
# sudo bin/run.sh --port /dev/ttyUSB0 --setch-shortfast
# sudo bin/run.sh --port /dev/ttyUSB0 --tunnel --debug
# ssh -Y root@192.168.10.151 (or dietpi), default password p
# ncat -e /bin/cat -k -u -l 1235
# ncat -u 10.115.64.152 1235
# ping -c 1 -W 20 10.115.64.152
# ping -i 30 -W 30 10.115.64.152

# FIXME: use a more optimal MTU
"""

import logging
import platform
import threading

from pubsub import pub # type: ignore[import-untyped]
from pytap2 import TapDevice

from meshtastic.protobuf import portnums_pb2
from meshtastic import mt_config
from meshtastic.util import ipstr, readnet_u16


def onTunnelReceive(packet, interface):  # pylint: disable=W0613
    """Callback for received tunneled messages from mesh."""
    logging.debug(f"in onTunnelReceive()")
    tunnelInstance = mt_config.tunnelInstance
    tunnelInstance.onReceive(packet)


class Tunnel:
    """A TUN based IP tunnel over meshtastic"""

    class TunnelError(Exception):
        """An exception class for general tunnel errors"""
        def __init__(self, message):
            self.message = message
            super().__init__(self.message)

    def __init__(self, iface, subnet: str="10.115", netmask: str="255.255.0.0") -> None:
        """
        Constructor

        iface is the already open MeshInterface instance
        subnet is used to construct our network number (normally 10.115.x.x)
        """

        if not iface:
            raise Tunnel.TunnelError("Tunnel() must have a interface")

        if not subnet:
            raise Tunnel.TunnelError("Tunnel() must have a subnet")

        if not netmask:
            raise Tunnel.TunnelError("Tunnel() must have a netmask")

        self.iface = iface
        self.subnetPrefix = subnet

        if platform.system() != "Linux":
            raise Tunnel.TunnelError("Tunnel() can only be run instantiated on a Linux system")

        mt_config.tunnelInstance = self

        """A list of chatty UDP services we should never accidentally
        forward to our slow network"""
        self.udpBlacklist = {
            1900,  # SSDP
            5353,  # multicast DNS
            9001,  # Yggdrasil multicast discovery
            64512, # cjdns beacon
        }

        """A list of TCP services to block"""
        self.tcpBlacklist = {
            5900,  # VNC (Note: Only adding for testing purposes.)
        }

        """A list of protocols we ignore"""
        self.protocolBlacklist = {
            0x02,  # IGMP
            0x80,  # Service-Specific Connection-Oriented Protocol in a Multilink and Connectionless Environment
        }

        # A new non standard log level that is lower level than DEBUG
        self.LOG_TRACE = 5

        # TODO: check if root?
        logging.info(
            "Starting IP to mesh tunnel (you must be root for this *pre-alpha* "
            "feature to work).  Mesh members:"
        )

        pub.subscribe(onTunnelReceive, "meshtastic.receive.data.IP_TUNNEL_APP")
        myAddr = self._nodeNumToIp(self.iface.myInfo.my_node_num)

        if self.iface.nodes:
            for node in self.iface.nodes.values():
                nodeId = node["user"]["id"]
                ip = self._nodeNumToIp(node["num"])
                logging.info(f"Node { nodeId } has IP address { ip }")

        logging.debug("creating TUN device with MTU=200")
        # FIXME - figure out real max MTU, it should be 240 - the overhead bytes for SubPacket and Data
        self.tun = None
        if self.iface.noProto:
            logging.warning(
                f"Not creating a TapDevice() because it is disabled by noProto"
            )
        else:
            self.tun = TapDevice(name="mesh")
            self.tun.up()
            self.tun.ifconfig(address=myAddr, netmask=netmask, mtu=200)

        self._rxThread = None
        if self.iface.noProto:
            logging.warning(
                f"Not starting TUN reader because it is disabled by noProto"
            )
        else:
            logging.debug(f"starting TUN reader, our IP address is {myAddr}")
            self._rxThread = threading.Thread(
                target=self.__tunReader, args=(), daemon=True
            )
            self._rxThread.start()

    def onReceive(self, packet):
        """onReceive"""
        p = packet["decoded"]["payload"]
        if packet["from"] == self.iface.myInfo.my_node_num:
            logging.debug("Ignoring message we sent")
        else:
            logging.debug(f"Received mesh tunnel message type={type(p)} len={len(p)}")
            # we don't really need to check for filtering here (sender should have checked),
            # but this provides useful debug printing on types of packets received
            if not self.iface.noProto:
                if not self._shouldFilterPacket(p):
                    self.tun.write(p)

    def _shouldFilterPacket(self, p):
        """Given a packet, decode it and return true if it should be ignored"""
        protocol = p[8 + 1]
        srcaddr = p[12:16]
        destAddr = p[16:20]
        subheader = 20
        ignore = False  # Assume we will be forwarding the packet
        if protocol in self.protocolBlacklist:
            ignore = True
            logging.log(
                self.LOG_TRACE, f"Ignoring blacklisted protocol 0x{protocol:02x}"
            )
        elif protocol == 0x01:  # ICMP
            icmpType = p[20]
            icmpCode = p[21]
            checksum = p[22:24]
            # pylint: disable=line-too-long
            logging.debug(
                f"forwarding ICMP message src={ipstr(srcaddr)}, dest={ipstr(destAddr)}, type={icmpType}, code={icmpCode}, checksum={checksum}"
            )
            # reply to pings (swap src and dest but keep rest of packet unchanged)
            # pingback = p[:12]+p[16:20]+p[12:16]+p[20:]
            # tap.write(pingback)
        elif protocol == 0x11:  # UDP
            srcport = readnet_u16(p, subheader)
            destport = readnet_u16(p, subheader + 2)
            if destport in self.udpBlacklist:
                ignore = True
                logging.log(self.LOG_TRACE, f"ignoring blacklisted UDP port {destport}")
            else:
                logging.debug(f"forwarding udp srcport={srcport}, destport={destport}")
        elif protocol == 0x06:  # TCP
            srcport = readnet_u16(p, subheader)
            destport = readnet_u16(p, subheader + 2)
            if destport in self.tcpBlacklist:
                ignore = True
                logging.log(self.LOG_TRACE, f"ignoring blacklisted TCP port {destport}")
            else:
                logging.debug(f"forwarding tcp srcport={srcport}, destport={destport}")
        else:
            logging.warning(
                f"forwarding unexpected protocol 0x{protocol:02x}, "
                "src={ipstr(srcaddr)}, dest={ipstr(destAddr)}"
            )

        return ignore

    def __tunReader(self):
        tap = self.tun
        logging.debug("TUN reader running")
        while True:
            p = tap.read()
            # logging.debug(f"IP packet received on TUN interface, type={type(p)}")
            destAddr = p[16:20]

            if not self._shouldFilterPacket(p):
                self.sendPacket(destAddr, p)

    def _ipToNodeId(self, ipAddr):
        # We only consider the last 16 bits of the nodenum for IP address matching
        ipBits = ipAddr[2] * 256 + ipAddr[3]

        if ipBits == 0xFFFF:
            return "^all"

        for node in self.iface.nodes.values():
            nodeNum = node["num"] & 0xFFFF
            # logging.debug(f"Considering nodenum 0x{nodeNum:x} for ipBits 0x{ipBits:x}")
            if (nodeNum) == ipBits:
                return node["user"]["id"]
        return None

    def _nodeNumToIp(self, nodeNum):
        return f"{self.subnetPrefix}.{(nodeNum >> 8) & 0xff}.{nodeNum & 0xff}"

    def sendPacket(self, destAddr, p):
        """Forward the provided IP packet into the mesh"""
        nodeId = self._ipToNodeId(destAddr)
        if nodeId is not None:
            logging.debug(
                f"Forwarding packet bytelen={len(p)} dest={ipstr(destAddr)}, destNode={nodeId}"
            )
            self.iface.sendData(p, nodeId, portnums_pb2.IP_TUNNEL_APP, wantAck=False)
        else:
            logging.warning(
                f"Dropping packet because no node found for destIP={ipstr(destAddr)}"
            )

    def close(self):
        """Close"""
        self.tun.close()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: meshtastic/protobuf/apponly.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from meshtastic.protobuf import channel_pb2 as meshtastic_dot_protobuf_dot_channel__pb2
from meshtastic.protobuf import config_pb2 as meshtastic_dot_protobuf_dot_config__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n!meshtastic/protobuf/apponly.proto\x12\x13meshtastic.protobuf\x1a!meshtastic/protobuf/channel.proto\x1a meshtastic/protobuf/config.proto\"\x81\x01\n\nChannelSet\x12\x36\n\x08settings\x18\x01 \x03(\x0b\x32$.meshtastic.protobuf.ChannelSettings\x12;\n\x0blora_config\x18\x02 \x01(\x0b\x32&.meshtastic.protobuf.Config.LoRaConfigBb\n\x13\x63om.geeksville.meshB\rAppOnlyProtosZ\"github.com/meshtastic/go/generated\xaa\x02\x14Meshtastic.Protobufs\xba\x02\x00\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'meshtastic.protobuf.apponly_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  DESCRIPTOR._serialized_options = b'\n\023com.geeksville.meshB\rAppOnlyProtosZ\"github.com/meshtastic/go/generated\252\002\024Meshtastic.Protobufs\272\002\000'
  _globals['_CHANNELSET']._serialized_start=128
  _globals['_CHANNELSET']._serialized_end=257
# @@protoc_insertion_point(module_scope)



"""Version lookup utilities, isolated for cleanliness"""
import sys
try:
    from importlib.metadata import version
except:
    import pkg_resources

def get_active_version():
    """Get the currently active version using importlib, or pkg_resources if we must"""
    if "importlib.metadata" in sys.modules:
        return version("meshtastic")
    else:
        return pkg_resources.get_distribution("meshtastic").version # pylint: disable=E0601



# delete me eventually
# Note python-pytuntap was too buggy
# using pip3 install pytap2
# make sure to "sudo setcap cap_net_admin+eip /usr/bin/python3.8" so python can access tun device without being root
# sudo ip tuntap del mode tun tun0

# FIXME: set MTU correctly
# select local ip address based on nodeid
# print known node ids as IP addresses

import logging
from _thread import start_new_thread

from pytap2 import TapDevice

"""A list of chatty UDP services we should never accidentally
forward to our slow network"""
udpBlacklist = {
    1900,  # SSDP
    5353,  # multicast DNS
}

"""A list of TCP services to block"""
tcpBlacklist = {}

"""A list of protocols we ignore"""
protocolBlacklist = {
    0x02,  # IGMP
    0x80,  # Service-Specific Connection-Oriented Protocol in a Multilink and Connectionless Environment
}


def hexstr(barray):
    """Print a string of hex digits"""
    return ":".join("{:02x}".format(x) for x in barray)


def ipstr(barray):
    """Print a string of ip digits"""
    return ".".join("{}".format(x) for x in barray)


def readnet_u16(p, offset):
    """Read big endian u16 (network byte order)"""
    return p[offset] * 256 + p[offset + 1]


def readtest(tap):
    while True:
        p = tap.read()

        protocol = p[8 + 1]
        srcaddr = p[12:16]
        destaddr = p[16:20]
        subheader = 20
        ignore = False  # Assume we will be forwarding the packet
        if protocol in protocolBlacklist:
            ignore = True
            logging.debug(f"Ignoring blacklisted protocol 0x{protocol:02x}")
        elif protocol == 0x01:  # ICMP
            logging.warn("Generating fake ping reply")
            # reply to pings (swap src and dest but keep rest of packet unchanged)
            pingback = p[:12] + p[16:20] + p[12:16] + p[20:]
            tap.write(pingback)
        elif protocol == 0x11:  # UDP
            srcport = readnet_u16(p, subheader)
            destport = readnet_u16(p, subheader + 2)
            logging.debug(f"udp srcport={srcport}, destport={destport}")
            if destport in udpBlacklist:
                ignore = True
                logging.debug(f"ignoring blacklisted UDP port {destport}")
        elif protocol == 0x06:  # TCP
            srcport = readnet_u16(p, subheader)
            destport = readnet_u16(p, subheader + 2)
            logging.debug(f"tcp srcport={srcport}, destport={destport}")
            if destport in tcpBlacklist:
                ignore = True
                logging.debug(f"ignoring blacklisted TCP port {destport}")
        else:
            logging.warning(
                f"unexpected protocol 0x{protocol:02x}, src={ipstr(srcaddr)}, dest={ipstr(destaddr)}"
            )

        if not ignore:
            logging.debug(
                f"Forwarding packet bytelen={len(p)} src={ipstr(srcaddr)}, dest={ipstr(destaddr)}"
            )


logging.basicConfig(level=logging.DEBUG)

tun = TapDevice(mtu=200)
# tun.create()
tun.up()
tun.ifconfig(address="10.115.1.2", netmask="255.255.0.0")

start_new_thread(readtest, (tun,))
input("press return key to quit!")
tun.close()
